# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk, filedialog
from tkinter.messagebox import showerror
import subprocess
import os
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as FigureCanvas
from shutil import copyfile
from voxplot import results, helpers



class Window(tk.Toplevel):
	'''Run analysis, update progress bar, and generate text and graphical output.

	:param parent: Object, reference, parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	:param samples: String, type of sample(s) (sv/cs/mx)
	:param indices: Integer, compute indices yes/no
	:param sv_ok: Integer, sv = 3 sec. yes/no
	:param metadata: Dict, input form data
	'''
	def __init__(self, parent, ASet, USet, samples, indices, sv_ok, metadata):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate
		# mimic grab_set() allowing minimize on Windows:
		helpers.toggle_root_state(self.aset.root, 'disabled')
		self.geometry('+{}+{}'.format(0, 0))
		self.protocol('WM_DELETE_WINDOW', self.close_pressed)
		self.samples = samples
		self.indices = indices
		self.sv_ok = sv_ok
		self.params = None
		self.metadata = metadata
		self.undef_value = 1000000
		self.undef_value_praat = 2000000

		mainframe = ttk.Frame(self, padding=(10,10,10,10))
		mainframe.grid(column=0, row=0)

		if self.samples != 'cs' and self.sv_ok == 1:
			self.render_graph = True
			if self.aset.screen_height >= 1190:
				# 100% Din A4
				self.figure = Figure(constrained_layout=True, figsize=(7.48,10.9))
				# self.figure = Figure(constrained_layout=True, figsize=(8.27,11.7))
				### self.figure = Figure(tight_layout=True, figsize=(8.27,11.7))
				show_logo = True
				logo_pady = 150
				logo_size = 1
				logo_fsize = 24
			elif self.aset.screen_height >= 800:
				# 66% Din A4
				self.figure = Figure(constrained_layout=True, figsize=(4.98667,7.2667))
				# self.figure = Figure(constrained_layout=True, figsize=(5.5133,7.8))
				show_logo = True
				logo_pady = 40
				logo_size = 2
				logo_fsize = 12
			else:
				# 50% Din A4
				self.figure = Figure(constrained_layout=True, figsize=(3.74,5.45))
				# self.figure = Figure(constrained_layout=True, figsize=(4.134,5.847))
				show_logo = False

			self.pdf_frame = ttk.Frame(mainframe)
			self.canvas = FigureCanvas(self.figure, master=self.pdf_frame)
			self.canvas.get_tk_widget().grid(column=0, row=0)
			self.pdf_frame.grid(column=1, row=0, sticky='nswe', padx=(10,0))
		else:
			self.render_graph = False
			show_logo = False

		ana_frame = ttk.Frame(mainframe)
		ana_frame.grid(column=0, row=0, sticky='nw')
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=0)
		self.rowconfigure(1, weight=1)
		self.rowconfigure(2, weight=0)
		self.rowconfigure(3, weight=0)

		self.aset.style.configure("my.Horizontal.TProgressbar", foreground='green', background='green')
		self.prog_bar = ttk.Progressbar(
			ana_frame,
			style='my.Horizontal.TProgressbar',
			orient='horizontal',
			mode='determinate',
			maximum=12
			)
		self.prog_bar.grid(column=0, row=0, sticky='we', padx=5, pady=5)

		txt_results_frame = ttk.Frame(ana_frame)
		txt_results_frame.grid(column=0, row=1, sticky='nswe')
		txt_results_frame.columnconfigure(0, weight=1)
		txt_results_frame.columnconfigure(1, weight=0)
		txt_results_frame.rowconfigure(0, weight=1)
		self.results_text = tk.Text(
			txt_results_frame,
			width=62,
			height=33,
			wrap='word',
			font='TkFixedFont',
			relief='solid',
			padx=10,
			pady=10,
			background='#eee'
			)
		yscroll = ttk.Scrollbar(
			txt_results_frame,
			orient = 'vertical',
			command = self.results_text.yview
			)
		self.results_text['yscrollcommand'] = yscroll.set
		self.results_text.grid(column=0, row=0, pady=5, sticky='nswe')
		yscroll.grid(column=1, row=0, pady=5, sticky = 'ns')
		self.configure(cursor='watch')
		self.results_text.configure(cursor='watch')
		self.results_text.tag_configure(
			'norm',
			foreground='green',
			borderwidth=0,
			relief='flat'
			)
		self.results_text.tag_configure(
			'warn',
			foreground='red',
			borderwidth=0,
			relief='flat'
			)

		btn_frame = ttk.Frame(ana_frame)
		btn_frame.grid(column=0, row=2, sticky='we')
		btn_frame.columnconfigure(0, weight=1)
		btn_frame.columnconfigure(1, weight=0)
		btn_frame.columnconfigure(2, weight=0)
		btn_frame.columnconfigure(3, weight=0)
		self.close_btn = ttk.Button(
			btn_frame,
			text=_("Close"),
			command=self.close_pressed
			)
		self.close_btn.grid(column=0, row=0, sticky='w', padx=(0,5))

		self.saveas_btn = ttk.Menubutton(
			btn_frame,
			text=_("Save")
			)
		saveas_menu = tk.Menu(
			self.saveas_btn,
			tearoff=False
			)
		if self.render_graph:
			saveas_menu.add_command(label='PDF', command=lambda: self.savegraph_pressed('PDF'))
			saveas_menu.add_command(label='PNG', command=lambda: self.savegraph_pressed('PNG'))
			saveas_menu.add_command(label='JPG', command=lambda: self.savegraph_pressed('JPG'))
			saveas_menu.add_separator()
		saveas_menu.add_command(label=_("CSV New"), command=lambda: self.csv_pressed('w'))
		saveas_menu.add_command(label=_("CSV Append"), command=lambda: self.csv_pressed('a'))
		self.saveas_btn['menu'] = saveas_menu
		self.saveas_btn.grid(column=3, row=0, sticky='e')

		if self.render_graph and show_logo:
			self.logoimg = tk.PhotoImage(file=self.aset.logo_s).subsample(logo_size)
			self.logo = ttk.Label(
				ana_frame,
				text=self.aset.app_name,
				font=f'-family {self.aset.font_family} -size {logo_fsize} -weight bold',
				image=self.logoimg,
				compound='top'
				)
			self.logo.grid(column=0, row=3, pady=(logo_pady,0))

		self.close_btn.state(['disabled'])
		self.saveas_btn.state(['disabled'])

		if self.metadata['sex'] == 'f':
			self.metadata['title'] = _("Ms.") + " "
		elif self.metadata['sex'] == 'm':
			self.metadata['title'] = _("Mr.") + " "
		else:
			self.metadata['title'] = ""
		heading = _("Subject") + ': ' + self.metadata['subid'] + '\n' + self.metadata['title'] + self.metadata['name'] + ' (' + self.metadata['dob'] + ')\n' + self.metadata['notes'] + '\n' + _("Examination") + ': ' + self.metadata['date'] + ', ' + self.metadata['time'] + ' ' + _("by") + ': ' + self.metadata['examiner'] + '\n' + _("Analysis Language") + ': ' + self.aset.analysis_langs[self.uset.a_lang] + '\n\n'
		txtline = '{:17} {:>10} {:>9} {:>10} {:>10}\n'.format("Parameter", "Norm", "SV", "CS", "MX")
		self.results_text.configure(state='normal')
		self.results_text.delete('1.0', 'end')
		self.results_text.insert('1.0', heading)
		self.results_text.insert('end', txtline)
		self.results_text.insert('end', '-------------------------------------------------------------\n')
		endidx = self.results_text.index('end').split('.')
		self.headidx = int(endidx[0])-1

		self.update_idletasks()
		self.after(5, self.step)


	def step(self, passage = 0):
		'''Run Praat analysis
		
		Call Praat (script) stepwise; performs part of the analysis at each passage,
		updates progress bar, and prints results to results_text widget. At the end,
		this function calls itself to perform next passage until passage = 11.

		:params passage: Int, passage index
		'''
		_ = self.aset.translate
		normv = self.aset.norm_values
		self.prog_bar['value'] = passage + 1
		try:
			# call Praat (script); args must be strings
			subprocess.run([self.aset.praat_exe, '--run', '--no-pref-files', '--no-plugins', '--utf8', self.aset.praat_script, str(passage), self.samples, str(self.indices), self.aset.praat_tmp_file, self.aset.tmp_dir, self.aset.sv_file, self.aset.cs_file, self.aset.mx_file, self.aset.sv_analyzed_file, self.aset.cs_analyzed_file, self.aset.mx_analyzed_file, self.aset.sv_pitch_file, self.aset.cs_pitch_file], check=True)
		except subprocess.CalledProcessError as e:
			showerror(_("Error"), _("A fatal error occurred during analysis. Analysis will be aborted."))
			self.destroy()
			self.aset.app_win.clear_pressed(clearall=False)
			return

		# get results of current passage
		self.params = {}
		ptf = open(self.aset.praat_tmp_file, 'r')
		for line in ptf:
			line = line.strip()
			list = line.split(': ')
			# handle undefined values
			try:
				self.params[list[0]] = float(list[1])
			except:
				self.params[list[0]] = self.undef_value
		ptf.close()

		if passage == 1:
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Slope (dB)", normv['slopex'], self.params['sv_slope'], self.params['cs_slope'], self.params['mx_slope'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Tilt (dB)", normv['tiltx'], self.params['sv_tilt'], self.params['cs_tilt'], self.params['mx_tilt'])
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 2:
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("HF Noise (dB)", normv['hfnox'], self.params['sv_hfno'], self.params['cs_hfno'], self.params['mx_hfno'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("HNRd (dB)", normv['hnrdx'], self.params['sv_hnrd'], self.params['cs_hnrd'], self.params['mx_hnrd'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("H1H2 (dB)", normv['h1h2x'], self.params['sv_h1h2'], self.params['cs_h1h2'], self.params['mx_h1h2'])
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 3:
			txtline = '{:17} {:>10} {:10.2f} '.format("CPPS (dB)", normv['cppsx'], self.params['sv_cpps'])
			self.results_text.insert('end', txtline)
			self.replace_undef()
		elif passage == 4:
			txtline = '{:10.2f} '.format(self.params['cs_cpps'])
			self.results_text.insert('end', txtline)
			self.replace_undef()
		elif passage == 5:
			txtline = '{:10.2f}\n'.format(self.params['mx_cpps'])
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 6:
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Jitter local (%)", normv['jitterLocalx'], self.params['sv_jitterLocal'], self.params['cs_jitterLocal'], self.params['mx_jitterLocal'])
			self.results_text.insert('end', txtline)

			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Jitter ppq5 (%)", normv['jitterPPQ5x'], self.params['sv_jitterPPQ5'], self.params['cs_jitterPPQ5'], self.params['mx_jitterPPQ5'])
			self.results_text.insert('end', txtline)

			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Shimmer (%)", normv['shimmerLocalx'], self.params['sv_shimmerLocal'], self.params['cs_shimmerLocal'], self.params['mx_shimmerLocal'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("Shimmer (dB)", normv['shimmerLocaldBx'], self.params['sv_shimmerLocaldB'], self.params['cs_shimmerLocaldB'], self.params['mx_shimmerLocaldB'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("HNR (dB)", normv['hnrx'], self.params['sv_hnr'], self.params['cs_hnr'], self.params['mx_hnr'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:10.2f}\n'.format("PSD (ms)", normv['psdx'], self.params['sv_psd'], self.params['cs_psd'], self.params['mx_psd'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:>10} {:>10}\n'.format("Voice Breaks", normv['voiceBreaksx'], self.params['sv_voiceBreaks'], '-', '-')
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 7:
			txtline = '{:17} {:>10} {:10.2f} '.format("GNE", normv['gnex'], self.params['sv_gne'])
			self.results_text.insert('end', txtline)
			self.replace_undef()
		elif passage == 8:
			txtline = '{:10.2f} '.format(self.params['cs_gne'])
			self.results_text.insert('end', txtline)
			self.replace_undef()
		elif passage == 9:
			txtline = '{:10.2f}\n'.format(self.params['mx_gne'])
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 10:
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:>10}\n'.format("Pitch (mean, Hz)", normv['meanPitchx'], self.params['sv_meanPitch'], self.params['cs_meanPitch'], '-')
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:>10}\n'.format("Pitch (min, Hz)", normv['minimumPitchx'], self.params['sv_minimumPitch'], self.params['cs_minimumPitch'], '-')
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:>10}\n'.format("Pitch (max, Hz)", normv['maximumPitchx'], self.params['sv_maximumPitch'], self.params['cs_maximumPitch'], '-')
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:>10}\n'.format("Pitch (sd, Hz)", normv['sdPitchx'], self.params['sv_sdPitch'], self.params['cs_sdPitch'], '-')
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:10.2f} {:10.2f} {:>10}\n'.format("Range (st)", normv['semitonesx'], self.params['sv_semitones'], self.params['cs_semitones'], '-')
			self.results_text.insert('end', txtline)
			self.replace_undef()

		elif passage == 11:
			txtline = '{:17} {:>10} {:>10} {:>10} {:10.2f}\n'.format("AVQI", normv['avqix'], '', '', self.params['avqi'])
			self.results_text.insert('end', txtline)
			txtline = '{:17} {:>10} {:>10} {:>10} {:10.2f}\n'.format("ABI", normv['abix'], '', '', self.params['abi'])
			self.results_text.insert('end', txtline)
			self.replace_undef()
			self.apply_tags()
			if self.params['resampled'] == 1:
				self.results_text.insert('end', '\n' + _("(CS was down-sampled to match sampling rate of SV)"))
			elif self.params['resampled'] == 2:
				self.results_text.insert('end', '\n' + _("(SV was down-sampled to match sampling rate of CS)"))
			elif self.sv_ok == 0:
				if self.aset.sv_too_long:
					self.results_text.insert('end', '\n' + _("(SV was too long; analysis incomplete.)"))
				elif self.aset.sv_too_short:
					self.results_text.insert('end', '\n' + _("(SV was too short; analysis incomplete.)"))

			# we're finished
			self.results_text.configure(state='disabled')
			self.close_btn.state(['!disabled'])
			self.saveas_btn.state(['!disabled'])
			if self.render_graph:
				self.update_idletasks()
				results.visualize(self.aset, self.uset.a_lang, self.figure, self.metadata, self.params, self.samples)
				self.figure.canvas.draw()
				self.figure.savefig(self.aset.pdf_file, format='pdf', bbox_inches='tight')
				self.figure.savefig(self.aset.png_file, format='png', dpi=150, bbox_inches='tight', pad_inches=0.5)
				self.figure.savefig(self.aset.jpg_file, format='jpg', dpi=150, bbox_inches='tight', pad_inches=0.5)

			self.configure(cursor='arrow')
			self.results_text.configure(cursor='arrow')
			return

		# run again to perform next passage
		self.after(5, lambda: self.step(passage+1))


	def csv_pressed(self, filemode):
		_ = self.aset.translate
		# specify field and decimal separators
		# field separator must be replaced within fields with replacement
		if self.uset.csv_field_delim == 0: # comma
			cfd = ','
			replacement = ';'
		elif self.uset.csv_field_delim == 1: # semi-colon
			cfd = ';'
			replacement = ','
		elif self.uset.csv_field_delim == 2: # TAB
			cfd = '\t'
			replacement = ','
		if self.uset.csv_decimal_sep == 0: # point
			cds = '.'
		elif self.uset.csv_decimal_sep == 1: # comma
			cds = ','

		heading = "name" + cfd + \
		"id" + cfd + \
		"dob" + cfd + \
		"sex" + cfd + \
		"notes" + cfd + \
		"analysis_language" + cfd + \
		"examiner" + cfd + \
		"exam_date" + cfd + \
		"exam_time" + cfd + \
		"cs_recording_date" + cfd + \
		"sv_recording_date"

		data = self.metadata['name'].replace(cfd, replacement) + cfd + \
			self.metadata['subid'].replace(cfd, replacement) + cfd + \
			self.metadata['sex'].replace(cfd, replacement) + cfd + \
			self.metadata['dob'].replace(cfd, replacement) + cfd + \
			self.metadata['notes'].replace('\n', ' ').replace(cfd, replacement) + cfd + \
			self.aset.analysis_langs[self.uset.a_lang].replace(cfd, replacement) + cfd + \
			self.metadata['examiner'].replace(cfd, replacement) + cfd + \
			self.metadata['date'].replace(cfd, replacement) + cfd + \
			self.metadata['time'].replace(cfd, replacement) + cfd + \
			self.metadata['csdate'].replace(cfd, replacement) + cfd + \
			self.metadata['svdate'].replace(cfd, replacement)

		for key in self.params:
			heading = heading + cfd + key
			value = (self.params[key] if self.params[key] < self.undef_value else '')
			svalue = str(value).replace('.', cds)
			data = data + cfd + svalue

		if filemode == 'w':
			response = filedialog.asksaveasfilename(
				title=_("Save as CSV file"),
				defaultextension='.csv',
				confirmoverwrite=True
				)
			if response != () and response != "":
				file = open(response, filemode)
				file.writelines([heading, "\n", data])
				file.close()
		elif filemode == 'a':
			response = filedialog.asksaveasfilename(
				title=_("Append to CSV file"),
				defaultextension='.csv',
				filetypes=(('CSV files', '*.csv'),),
				confirmoverwrite=False
				)
			if response != () and response != "":
				file = open(response, filemode)
				file.writelines(["\n", data])
				file.close()
		self.lift(self.aset.root)


	def savegraph_pressed(self, file_format):
		_ = self.aset.translate
		if file_format == 'PDF':
			target_file = self.aset.pdf_file
			extension = '.pdf'
		elif file_format == 'PNG':
			target_file = self.aset.png_file
			extension = '.png'
		elif file_format == 'JPG':
			target_file = self.aset.jpg_file
			extension = '.jpg'

		if os.path.exists(target_file):
			response = filedialog.asksaveasfilename(
				title=_("Save as {} file").format(file_format),
				defaultextension=extension,
				confirmoverwrite=True
				)
			if response != () and response != "":
				try:
					copyfile(target_file, response)
				except:
					showerror(_("Save {} file failed").format(file_format),
						_("Saving {} file to the following destination failed:").format(file_format) + '\n' + response
						)
			self.lift(self.aset.root)
		else:
			showerror(_("{} file missing").format(file_format),
				_("{} file creation failed. Can't save.").format(file_format)
				)


	def close_pressed(self, *args):
		helpers.toggle_root_state(self.aset.root)
		self.destroy()


	def replace_undef(self):
		# search and replace undefined values
		while True:
			idx = self.results_text.search(str(self.undef_value) + '.00000', '1.0', 'end')
			if not idx: break
			self.results_text.delete(idx, idx + ' + 13c')
			self.results_text.insert(idx, '     undef   ')
		while True:
			idx = self.results_text.search(str(self.undef_value) + '.00', '1.0', 'end')
			if not idx: break
			self.results_text.delete(idx, idx + ' + 10c')
			self.results_text.insert(idx, '     undef')
		while True:
			idx = self.results_text.search(str(self.undef_value_praat) + '.00', '1.0', 'end')
			if not idx: break
			self.results_text.delete(idx, idx + ' + 10c')
			self.results_text.insert(idx, '         -')


	def apply_tags(self):
		normv = self.aset.norm_values
		uvp = self.undef_value_praat
		for line in range(self.headidx, self.headidx + 21, 1):
			# remove norm values if sv sample is missing
			if self.samples == 'cs' or self.sv_ok == 0:
				self.results_text.delete(str(line) + '.18', str(line) + '.28')
				self.results_text.insert(str(line) + '.18', '         -')
			else:
				# tag norm values
				self.results_text.tag_add('norm', str(line) + '.18', str(line) + '.28')
				# tag deviations from norm
				warn = False
				start = self.results_text.get(str(line)+'.1 wordstart', str(line)+'.17')
				if start.startswith('Slope (dB)'): pass
				elif start.startswith('Tilt (dB)'): pass
				elif start.startswith('HF Noise (dB)') and normv['hfno'] != None and self.params['sv_hfno'] < normv['hfno'] and self.params['sv_hfno'] < uvp: warn = True
				elif start.startswith('HNRd (dB)') and normv['hnrd'] != None and self.params['sv_hnrd'] < normv['hnrd'] and self.params['sv_hnrd'] < uvp: warn = True
				elif start.startswith('H1H2 (dB)'): pass
				elif start.startswith('CPPS (dB)') and normv['cpps'] != None and self.params['sv_cpps'] < normv['cpps'] and self.params['sv_cpps'] < uvp: warn = True
				elif start.startswith('Jitter local (%)') and normv['jitterLocal'] != None and self.params['sv_jitterLocal'] > normv['jitterLocal'] and self.params['sv_jitterLocal'] < uvp: warn = True
				elif start.startswith('Jitter ppq5 (%)') and normv['jitterPPQ5'] != None and self.params['sv_jitterPPQ5'] > normv['jitterPPQ5'] and self.params['sv_jitterPPQ5'] < uvp: warn = True
				elif start.startswith('Shimmer (%)') and normv['shimmerLocal'] != None and self.params['sv_shimmerLocal'] > normv['shimmerLocal'] and self.params['sv_shimmerLocal'] < uvp: warn = True
				elif start.startswith('Shimmer (dB)') and normv['shimmerLocaldB'] != None and self.params['sv_shimmerLocaldB'] > normv['shimmerLocaldB'] and self.params['sv_shimmerLocaldB'] < uvp: warn = True
				elif start.startswith('HNR (dB)') and normv['hnr'] != None and self.params['sv_hnr'] < normv['hnr'] and self.params['sv_hnr'] < uvp: warn = True
				elif start.startswith('PSD (ms)') and normv['psd'] != None and self.params['sv_psd'] > normv['psd'] and self.params['sv_psd'] < uvp: warn = True
				elif start.startswith('Voice Breaks') and normv['voiceBreaks'] != None and self.params['sv_voiceBreaks'] > normv['voiceBreaks'] and self.params['sv_voiceBreaks'] < uvp: warn = True
				elif start.startswith('GNE') and normv['gne'] != None and self.params['sv_gne'] < normv['gne'] and self.params['sv_gne'] < uvp: warn = True
				elif start.startswith('Pitch (mean, Hz)'): pass
				elif start.startswith('Pitch (min, Hz)'): pass
				elif start.startswith('Pitch (max, Hz)'): pass
				elif start.startswith('Pitch (sd, Hz)'): pass
				elif start.startswith('Range (st)'): pass
				if warn: self.results_text.tag_add('warn', str(line) + '.29', str(line) + '.39')
				warn = False
				if start.startswith('AVQI') and normv['avqi'] != None and self.params['avqi'] > normv['avqi'] and self.params['avqi'] < uvp: warn = True
				elif start.startswith('ABI') and normv['abi'] != None and self.params['abi'] > normv['abi'] and self.params['abi'] < uvp: warn = True
				if warn: self.results_text.tag_add('warn', str(line) + '.51', str(line) + '.61')

				# remove AVQI and ABI norm values if cs sample is missing
				if (start.startswith('AVQI') or start.startswith('ABI')) and self.samples == 'sv':
					self.results_text.delete(str(line) + '.18', str(line) + '.28')
					self.results_text.insert(str(line) + '.18', '         -')





