# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
import numpy as np
import threading
import os
import time
import wave
import pyaudio
from pydub import AudioSegment

### for Plotter class
from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as FigureCanvas
from matplotlib.patches import Rectangle




class Window(tk.Toplevel):
	'''Handle recording of noise and signal as well as SNR estimation.

	:param parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	'''
	def __init__(self, parent, ASet, USet):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.grab_set()
		### topmost=1 prevents messagebox display on top...!
		self.attributes('-topmost', 1)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate
		self.title(_("Evaluate Signal-to-Noise Ratio"))
		self.recdur = 3
		self.type = ''
		self.noise_db = 0.0
		self.signal_db = 0.0
		
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)

		### frames and widgets
		mainframe = ttk.Frame(self, padding=(10,10,10,10))
		mainframe.grid(column=0, row=0)
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)
		self.rowconfigure(1, weight=1)
		self.rowconfigure(2, weight=1)
		self.rowconfigure(3, weight=0)
		self.rowconfigure(4, weight=0)

		# heading
		heading_frame = ttk.Frame(mainframe)
		heading_frame.grid(column=0, row=0, sticky='ns')
		self.heading_label = ttk.Label(
			heading_frame,
			text=_("Signal-to-Noise Estimation in 2 Steps"),
			font=f'-family {self.aset.font_family} -size 16 -weight bold'
			)
		self.heading_label.grid(column=0, row=0, pady=(0,20))

		# reading
		read_frame = ttk.Frame(mainframe)
		read_frame.grid(column=0, row=1, sticky='ns')
		self.read_label = ttk.Label(
			read_frame,
			text=_("Step 1: Noise evaluation (ambient noise and noise from the recording equipment)"),
			font=f'-family {self.aset.font_family} -size 14'
			)
		self.read_label.grid(column=0, row=0, sticky='w', pady=(0,10))
		self.instruc_label = ttk.Label(
			read_frame,
			text=_("Start the recording (press 1st button or type <1>) and be completely silent.") + "\n" + _("Do not breathe into the microphone.") + "\n" + _("The recording ends automatically after 3 sec."),
			font=f'-family {self.aset.font_family} -size 14 -slant italic'
			)
		self.instruc_label.grid(column=0, row=1, sticky='w', pady=(0,20))

		# device info
		rec_dev = _("Recording device") + f": {self.aset.audio_info['name']} ({str(self.aset.sampling_rate)} Hz)"
		self.dev_info_label = ttk.Label(
			read_frame,
			text=rec_dev,
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size - 1)}',
			)
		self.dev_info_label.grid(column=0, row=2, pady=(0,0))

		# plots
		self.aset.style.configure("my.Horizontal.TProgressbar", foreground='green', background='green')
		plot_boxes = ttk.Frame(mainframe)
		plot_boxes.grid(column=0, row=3, pady=(0,40))

		self.plot_box_1 = ttk.Frame(plot_boxes)
		self.plot_box_1.grid(column=0, row=0,)
		self.pb_noise = ttk.Progressbar(
			plot_boxes,
			style='my.Horizontal.TProgressbar',
			orient='horizontal',
			mode='determinate',
			maximum=100,
		)
		self.pb_noise.grid(column=0, row=1, padx=(60,60), pady=(1,5), sticky='we')
		self.noise_btn = ttk.Button(
			plot_boxes,
			text=_("Step 1: Noise evaluation"),
			command=lambda: self.start_recording('noise')
			)
		self.noise_btn.grid(column=0, row=2, sticky='we', padx=(60,60), pady=(10,0))

		self.plot_box_2 = ttk.Frame(plot_boxes)
		self.plot_box_2.grid(column=1, row=0,)
		self.pb_signal = ttk.Progressbar(
			plot_boxes,
			style='my.Horizontal.TProgressbar',
			orient='horizontal',
			mode='determinate',
			maximum=100,
		)
		self.pb_signal.grid(column=1, row=1, padx=(60,60), pady=(1,5), sticky='we')
		self.signal_btn = ttk.Button(
			plot_boxes,
			text=_("Step 2: Vowel /aaa/"),
			command=lambda: self.start_recording('signal')
			)
		self.signal_btn.grid(column=1, row=2, sticky='we', padx=(60,60), pady=(10,0))

		# buttons
		button_frame = ttk.Frame(mainframe)
		button_frame.grid(column=0, row=4, sticky='nse', pady=(0,0))
		button_frame.rowconfigure(0, weight=1)
		self.repeat_btn = ttk.Button(
			button_frame,
			text=_("Repeat"),
			command=self.repeat_pressed
			)
		self.repeat_btn.grid(column=4, row=0, sticky='e', padx=(0,5))
		self.cancel_btn = ttk.Button(
			button_frame,
			text=_("Cancel"),
			command=self.cancel_pressed
			)
		self.cancel_btn.grid(column=5, row=0, sticky='e')


		### initializations and bindings
		self.signalplot_1 = None
		self.signalplot_2 = None
		self.rec_thread = None
		self.rec_exit_event = threading.Event()
		self.protocol('WM_DELETE_WINDOW', self.cancel_pressed)

		self.signalplot_1 = Plotter('noise', self.recdur, self.plot_box_1, self.aset, self.uset, large=True)
		self.signalplot_1.init_plot()
		self.signalplot_2 = Plotter('signal', self.recdur, self.plot_box_2, self.aset, self.uset, large=True)
		self.signalplot_2.init_plot()

		self.bind('<KeyPress-Escape>', self.cancel_pressed)
		# remove the default behavior of invoking the button with the space key
		self.unbind_class('Button', '<Key-space>')
		self.bind('<KeyPress-1>', lambda event: self.start_recording('noise'))
		self.bind('<KeyPress-2>', lambda event: self.start_recording('signal'))
		self.signal_btn.state(['disabled'])
		self.repeat_btn.state(['disabled'])


	def start_recording(self, type, *args):
		_ = self.aset.translate
		self.rec_exit_event.clear()
		self.type = type
		if self.type == 'noise':
			self.noise_btn['text'] = _("Recording")
			# clear and reset plot
			self.signalplot_1.init_plot()
			self.signalplot_1.figure.canvas.draw()
		elif self.type == 'signal':
			self.signal_btn['text'] = _("Recording")
			# clear and reset plot
			self.signalplot_2.init_plot()
			self.signalplot_2.figure.canvas.draw()
		self.noise_btn.state(['disabled'])
		self.signal_btn.state(['disabled'])
		self.rec_thread = threading.Thread(target=self.recorder)
		self.rec_thread.start()
		# run progress bars
		pbv = 0
		while pbv <= 100:
			if self.type == 'noise':
				self.pb_noise['value'] = pbv
			elif self.type == 'signal':
				self.pb_signal['value'] = pbv
			time.sleep(0.3)
			pbv=pbv+10
			self.update_idletasks()


	def after_recording(self):
		_ = self.aset.translate
		self.rec_thread = None
		self.noise_btn['text'] = _("Step 1: Noise evaluation")
		self.signal_btn['text'] = _("Step 2: Vowel /aaa/")
		if self.type == 'noise':
			self.noise_btn.state(['disabled'])
			self.signal_btn.state(['!disabled'])
		elif self.type == 'signal':
			self.signal_btn.state(['disabled'])
			self.repeat_btn.state(['!disabled'])
			self.cancel_btn.configure(text = _("Close"))
		time.sleep(0.2)
		if self.type == 'noise':
			self.noise_db = self.signalplot_1.plot_sound(self.aset.tmp_file)
			self.read_label.configure(text = _("Step 2: Signal evaluation (the actual recording used for acoustic voice quality analysis)"))
			self.instruc_label.configure(text = _("Start the recording (press 2nd button or type <2>) and sustain the vowel /aaa/.") + "\n" + _("Speak at a comfortable volume like you would during examination.") + "\n" + _("The recording ends automatically after 3 sec."))
		elif self.type == 'signal':
			self.signal_db = self.signalplot_2.plot_sound(self.aset.tmp_file)
			# Calculate SNR
			snr = self.signal_db - self.noise_db
			if np.isnan(snr):
				self.read_label.configure(text = _("Something went wrong, we couldn't estimate SNR."))
				self.instruc_label.configure(text = _("\nPlease try again.\n"))
			else:
				snr_comment = ""
				if snr <= 30:
					snr_comment = _("SNR is too low!") + "\n" + _("SNR > 30 dB is required, SNR > 42 dB would be excellent.") + "\n" + _("Please try to reduce background noise and repeat the procedure.")
				elif snr < 42:
					snr_comment = _("SNR is acceptable.") + "\n" + _("SNR > 42 dB would be excellent.") + "\n" + _("To further improve your recording setup try to reduce background noise.")
				elif snr >= 42:
					snr_comment = "\n" + _("SNR is excellent.") + "\n" + _("Your noise level is well suited for acoustic voice quality analysis.")

				self.read_label.configure(text = _("estimated SNR:") + f" {snr:.2f} dB")
				self.instruc_label.configure(text = snr_comment)


	def recorder(self):
		fname = self.aset.tmp_file
		if os.path.exists(fname):
			os.remove(fname)
		channels = self.aset.channels
		srate = self.aset.sampling_rate
		sformat = pyaudio.paInt16
		chunk = self.aset.chunk
		pa = pyaudio.PyAudio()
		duration = self.recdur
		wavefile = wave.open(fname, 'wb')
		wavefile.setnchannels(channels)
		wavefile.setsampwidth(pa.get_sample_size(sformat))
		wavefile.setframerate(srate)
		stream = pa.open(
			format=sformat,
			channels=channels,
			rate=srate,
			input=True,
			frames_per_buffer=chunk
			)
		frames_done = 0
		frame_count = 0
		while frame_count <= int((srate / chunk) * duration) and not self.rec_exit_event.is_set():
			audio = stream.read(chunk)
			wavefile.writeframes(audio)
			frame_count += 1
		wavefile.close()
		stream.stop_stream()
		stream.close()
		pa.terminate()
		self.after_recording()


	def repeat_pressed(self, *args):
		_ = self.aset.translate
		self.noise_db = 0.0
		self.signal_db = 0.0
		self.signalplot_1 = Plotter('noise', self.recdur, self.plot_box_1, self.aset, self.uset, large=True)
		self.signalplot_1.init_plot()
		self.signalplot_2 = Plotter('signal', self.recdur, self.plot_box_2, self.aset, self.uset, large=True)
		self.signalplot_2.init_plot()
		self.pb_noise['value'] = 0
		self.pb_signal['value'] = 0
		self.noise_btn.state(['!disabled'])
		self.signal_btn.state(['disabled'])
		self.repeat_btn.state(['disabled'])
		self.cancel_btn.configure(text = _("Cancel"))
		self.read_label.configure(text = _("Step 1: Noise evaluation (ambient noise and noise from the recording equipment)"))
		self.instruc_label.configure(text = _("Start the recording (press 1st button or type <1>) and be completely silent.") + "\n" + _("Do not breathe into the microphone.") + "\n" + _("The recording ends automatically after 3 sec."))


	def cancel_pressed(self, *args):
		self.destroy()




class Plotter(object):
	'''Handle signal plots.

	:param type: String, type of signal (noise or signal)
	:param recdur: Number, recording duration (could be integer or float)
	:param target: Object, reference to target frame
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	:param large: Boolean, plot size large (True) or small (False)
	'''
	def __init__(self, type, recdur, target, ASet, USet, large=False):
		self.type = type
		self.aset = ASet
		self.uset = USet
		self.recdur = recdur
		if large:
			self.figure = Figure(tight_layout=True, figsize=(4.5,2))
		else:
			self.figure = Figure(tight_layout=True, figsize=(3,1.7))
		self.subplot = self.figure.add_subplot(111)
		self.canvas = FigureCanvas(self.figure, master=target)
		self.canvas.get_tk_widget().grid(column=0, row=0)
		xtick_labels_int = []
		self.xtick_labels = []


	def init_plot(self, rate=0):
		if rate == 0: rate = self.aset.sampling_rate
		self.subplot.cla()
		self.subplot.set_xlim(0, self.recdur * rate)
		self.subplot.set_ylim(-1, 1)
		seconds = int(self.recdur)
		xtick_labels_int = [l for l in range(0,seconds + 1,1)]
		xtick_labels = list(map(str, xtick_labels_int))
		self.subplot.set_xticks(range(0, (seconds + 1) * rate, rate))
		self.subplot.set_xticklabels(xtick_labels[0:seconds + 1])
		self.subplot.set_yticks([0])
		self.subplot.set_yticklabels(['0'])


	def plot_sound(self, sndfile):
		with wave.open(sndfile, 'rb') as wf:
			rate = wf.getframerate()
			gframes = wf.getnframes()
			nframes = rate * self.recdur
			nframes = (gframes if nframes > gframes else nframes)
			audio_data = np.frombuffer(wf.readframes(nframes), dtype=np.int16) / 32768.0
		self.init_plot(rate)
		self.subplot.plot(
			range(0, nframes, 1),
			audio_data,
			color=(0,0,0,0.5),
			zorder=5
			)
		self.xMin,self.xMax = self.subplot.get_xlim()
		self.yMin,self.yMax = self.subplot.get_ylim()
		start = round(rate)
		end = round(rate * 2.5)
		self.rect = Rectangle(
			(start, self.yMin),
			end - start,
			(self.yMin * -1) + self.yMax,
			linewidth=2,
			edgecolor='r',
			facecolor='none',
			zorder=100
			)
		self.subplot.add_patch(self.rect)
		self.figure.canvas.draw()
		if self.type == 'noise':
			noise_rms = np.sqrt(np.mean(audio_data[start:end] ** 2))
			noise_db = 20 * np.log10(noise_rms)
			return noise_db
		elif self.type == 'signal':
			signal_rms = np.sqrt(np.mean(audio_data[start:end] ** 2))
			signal_db = 20 * np.log10(signal_rms)
			return signal_db





