###############################
# Copyright Jörg Mayer
# 
# This file is part of VOXplot.
# 
# VOXplot is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# VOXplot is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with VOXplot.  If not, see <https://www.gnu.org/licenses/>.
# 
###############################


### get arguments from python call
form command line arguments
	integer passage 0
	word samples Empty
	boolean indices Empty
	sentence praat_tmp_file Empty
	sentence tmp_dir Empty
	sentence sv_file Empty
	sentence cs_file Empty
	sentence mx_file Empty
	sentence sv_analyzed_file Empty
	sentence cs_analyzed_file Empty
	sentence mx_analyzed_file Empty
	sentence sv_pitch_file Empty
	sentence cs_pitch_file Empty
endform

undef_value = 2000000





### PROCEDURES

# read stuff in passages > 0
procedure passageRead:
	# read sound files and assign IDs to global variables
	if samples$ = "cs" or samples$ = "mx"
		cs_sound = Read from file: cs_analyzed_file$
	endif
	if samples$ = "sv" or samples$ = "mx"
		sv_sound = Read from file: sv_analyzed_file$
	endif
	if samples$ = "mx" and indices
		mx_sound = Read from file: mx_analyzed_file$
	endif
	# read previous results
	.prev_results = Read Strings from raw text file: praat_tmp_file$
	.nos = Get number of strings
	for .n from 1 to .nos
		selectObject(.prev_results)
		.line$ = Get string: .n
		# extract variable names and values
		.var_name$ = replace_regex$(.line$, "(\w): .+", "\1", 0)
		.value = extractNumber (.line$, .var_name$ + ":")
		# assign value to global variable
		'.var_name$' = .value
	endfor
endproc


# extract and concatenate voiced segments of cs sample (main proc)
procedure getVoicedSegments: .sndid
	### continuous speech: extract and concatenate voiced segments
	### largely based on a script by Paul Corthals, Ghent University, Belgium
	###
	### main procedure
	selectObject: .sndid
	.samplingRate = Get sampling frequency
	.intermediateSamples = Get sampling period
	.ov = Create Sound: "ov", 0, 0.001, .samplingRate, "0"
	.uniqname$ = replace_regex$ (date$ (), "[ :]", "", 0)
	selectObject: .sndid
	.tg = To TextGrid (silences): 50, 0.003, -25, 0.1, 0.1, "", .uniqname$
	plusObject: .sndid
	Extract non-empty intervals: 1, "no"
	.conc = Concatenate
	selectObject: .conc
	.signalEnd = Get end time
	.windowBorderLeft = Get start time
	.windowWidth = 0.03
	.windowBorderRight = .windowBorderLeft + .windowWidth
	.extremeRight = .signalEnd - .windowWidth
	.globalPower = Get power in air
	.voicelessThreshold = .globalPower*(30/100)
	while .windowBorderRight < .extremeRight
		selectObject: .conc
		.part = Extract part: .windowBorderLeft, .windowBorderRight, "Rectangular", 1.0, "no"
		.partialPower = Get power in air
		if .partialPower > .voicelessThreshold
			@checkZeros: .part, 0, .intermediateSamples
			if (checkZeros.zeroCrossingRate <> undefined) and (checkZeros.zeroCrossingRate < 3000)
				selectObject: .ov, .part
				oldov = .ov
				.ov = Concatenate
				removeObject: oldov
			endif
		endif
		.windowBorderLeft = .windowBorderLeft + 0.03
		.windowBorderRight = .windowBorderLeft + 0.03
	endwhile
	.result = .ov
endproc


# extract and concatenate voiced segments of cs sample (helper proc)
procedure checkZeros: .sndid, .zeroCrossingRate, .isam
	### continuous speech: extract and concatenate voiced segments
	### largely based on a script by Paul Corthals, Ghent University, Belgium
	###
	### check zeros procedure
	selectObject: .sndid
	.start = 0.0025
	.startZero = Get nearest zero crossing: .start
	.findStart = .startZero
	.findStartZeroPlusOne = .startZero + .isam
	.startZeroPlusOne = Get nearest zero crossing: .findStartZeroPlusOne
	.zeroCrossings = 0
	.strips = 0
	while (.findStart < 0.0275) and (.findStart <> undefined)
		while .startZeroPlusOne = .findStart
			.findStartZeroPlusOne = .findStartZeroPlusOne + .isam
			.startZeroPlusOne = Get nearest zero crossing: .findStartZeroPlusOne
		endwhile
		.distance = .startZeroPlusOne - .startZero
		.strips = .strips + 1
		.zeroCrossings = .zeroCrossings + 1
		.findStart = .startZeroPlusOne
	endwhile
	.zeroCrossingRate = .zeroCrossings/.distance
endproc


# estimate optimal pitch analysis range (cf. Hirst, 2011)
procedure analysisRange: .sndid, .width
	selectObject: .sndid
	.p = To Pitch: 0.01, 60, 700
	.q1 = Get quantile: 0, 0, 0.25, "Hertz"
	.q3 = Get quantile: 0, 0, 0.75, "Hertz"
	if .q1 <> undefined and .q3 <> undefined
		.floor = round (.q1*0.75)
		if .floor < 60
			.floor = 60
		endif
		.ceiling = round (.q3 * .width)
	else
		.floor = 70
		.ceiling = 600
	endif
endproc


# CPPS
procedure getCPPS: .sndid
	selectObject: .sndid
	.pcep = To PowerCepstrogram: 60, 0.002, 5000, 50
	.result = Get CPPS: "no", 0.01, 0.001, 60, 330, 0.05, "Parabolic", 0.001, 0, "Straight", "Robust"
endproc


# Slope & Tilt
procedure getSlope: .sndid
	selectObject: .sndid
	.dur = Get total duration
	if .dur >= 0.8
		.ltas = To Ltas: 1
		.slope = Get slope: 0, 1000, 1000, 10000, "energy"
		.trend = Compute trend line: 1, 10000
		.tilt = Get slope: 0, 1000, 1000, 10000, "energy"
	elsif .dur >= 0.07
		.ltas = To Ltas: 12
		.slope = Get slope: 0, 1000, 1000, 10000, "energy"
		.trend = Compute trend line: 1, 10000
		.tilt = Get slope: 0, 1000, 1000, 10000, "energy"
	else
		.slope = undefined
		.tilt = undefined
	endif
endproc


# HFNo, HNRd & H1H2
procedure getABIparms: .sndid
	selectObject: .sndid
	.start = Get start time
	.duration = Get total duration
	.durationAnalysisWindow = 0.1
	.halfDurationAnalysisWindow = .durationAnalysisWindow / 2
	.numberOfWindows = floor (.duration / .durationAnalysisWindow)


	### Analyses on portions with durationAnalysisWindow
	# Intermediate tables
	.table1 = Create Table with column names: "hfno", 0, "hfno6000"
	.table2 = Create Table with column names: "hnrd", 0, "hnrd"
	.table3 = Create Table with column names: "h1h2", 0, "h1h2"

	# Segmentation into portions of 0.05 s, and analyses on these portions
	for .n from 1 to .numberOfWindows
		selectObject: .sndid
		.endWindow = .start + (.n * .durationAnalysisWindow)
		.startWindow = .endWindow - .durationAnalysisWindow
		.soundInWindow = Extract part: .startWindow, .endWindow, "rectangular", 1, "yes"
		
		# High Frequency Noise (hfno)
		.spec1 = To Spectrum: "yes"
		.ltas1 = To Ltas (1-to-1)
		.ltasMinimum = Get minimum: 0, 10000, "None"
		if .ltasMinimum < 0
			.ltasMinimum = abs (.ltasMinimum)
			Formula: "self + .ltasMinimum"
		elsif .ltasMinimum >= 0
			Formula: "self - .ltasMinimum"
		endif
		.ltasEnergy0to6kHz = Get mean: 0, 6000, "energy"
		.ltasEnergy6to10kHz = Get mean: 6000, 10000, "energy"
		.hfno6000 = .ltasEnergy0to6kHz / .ltasEnergy6to10kHz
		selectObject: .table1
		Append row
		Set numeric value: .n, "hfno6000", .hfno6000
		# meanhfno6000 = Get mean: "hfno6000"

		# HNR-Dejonckere (hnrd) (between 0.5-1.5 kHz) (after f0 determination: (a) cepstrum, (b) LTAS) 
		selectObject: .soundInWindow
		.startSoundInWindow = Get start time
		.middleSoundInWindow = .startSoundInWindow+.halfDurationAnalysisWindow
		.cep1 = To PowerCepstrogram: 60, 0.002, 5000, 50
		.cep2 = To PowerCepstrum (slice): .middleSoundInWindow
		.cep3 = Subtract tilt: 0.001, 0, "Straight", "Robust"
		.quefrencyPeak = Get quefrency of peak: 60, 400, "Parabolic"
		.frequencySoundInWindow = 1 / .quefrencyPeak
		.frequencySoundInWindowLowerSearchLimit = .frequencySoundInWindow - 20
		.frequencySoundInWindowUpperSearchLimit = .frequencySoundInWindow + 20
		selectObject: .spec1
		.ltas2 = To Ltas: 6
		.peakInFrequencyZone = Get frequency of maximum: .frequencySoundInWindowLowerSearchLimit, .frequencySoundInWindowUpperSearchLimit, "None"
		.firstHarmonicIn500to1500 = ceiling (500 / .peakInFrequencyZone)
		.lastHarmonicIn500to1500 = floor (1500 / .peakInFrequencyZone)
		.table4 = Create Table with column names: "peaks", 0, "peakHeight"
		selectObject: .ltas1
		for .p from .firstHarmonicIn500to1500 to .lastHarmonicIn500to1500
			selectObject: .ltas1
			.harmonicZoneLeftBoundary = (.p * .peakInFrequencyZone) - 20
			.harmonicZoneRightBoundary = (.p * .peakInFrequencyZone) + 20
			.peakInHarmonicZone = Get maximum: .harmonicZoneLeftBoundary, .harmonicZoneRightBoundary, "None"
			selectObject: .table4
			Append row
			.numberOfRows = Get number of rows
			Set numeric value: .numberOfRows, "peakHeight", .peakInHarmonicZone
		endfor
		selectObject: .table4
		.meanPeakHeight = Get mean: "peakHeight"
		.firstValleyIn500to1500 = .firstHarmonicIn500to1500
		.lastValleyIn500to1500 = (floor (1500 / .frequencySoundInWindow)) - 1
		.table5 = Create Table with column names: "valleys", 0, "valleyDepth"
		selectObject: .ltas1
		for .v from .firstValleyIn500to1500 to .lastValleyIn500to1500
			selectObject: .ltas1
			.valleyZoneLeftBoundary = (.v * .peakInFrequencyZone) + 20
			.valleyZoneRightBoundary = ((.v + 1) * .peakInFrequencyZone) - 20
			.depthInValleyZone = Get minimum: .valleyZoneLeftBoundary, .valleyZoneRightBoundary, "None"
			selectObject: .table5
			Append row
			.numberOfRows= Get number of rows
			Set numeric value: .numberOfRows, "valleyDepth", .depthInValleyZone
		endfor
		selectObject: .table5
		.meanValleyDepth = Get mean: "valleyDepth"
		.hnrd = .meanPeakHeight-.meanValleyDepth
		selectObject: .table2
		Append row
		Set numeric value: .n, "hnrd", .hnrd
		
		# h1-h2
		.firstHarmonicLowerBoundary = .peakInFrequencyZone - 20
		.firstHarmonicUpperBoundary = .peakInFrequencyZone + 20
		.h2PeakInFrequencyZone = .peakInFrequencyZone * 2
		.secondHarmonicLowerBoundary = .h2PeakInFrequencyZone - 20
		.secondHarmonicUpperBoundary = .h2PeakInFrequencyZone + 20
		selectObject: .ltas2
		.h1Amplitude = Get maximum: .firstHarmonicLowerBoundary, .firstHarmonicUpperBoundary, "None"
		.h2Amplitude = Get maximum: .secondHarmonicLowerBoundary, .secondHarmonicUpperBoundary, "None"
		.h1h2 = .h1Amplitude-.h2Amplitude
		selectObject: .table3
		Append row
		Set numeric value: .n, "h1h2", .h1h2

		nocheck removeObject: .soundInWindow
		nocheck removeObject: .spec1
		nocheck removeObject: .ltas1
		nocheck removeObject: .cep1
		nocheck removeObject: .cep2
		nocheck removeObject: .cep3
		nocheck removeObject: .ltas2
		nocheck removeObject: .table4
		nocheck removeObject: .table5
	endfor

	# Retrieve final results from intermediate tables
	selectObject: .table1
	.finalhfno6000 = Get mean: "hfno6000"
	selectObject: .table2
	.finalhnrd = Get mean: "hnrd"
	selectObject: .table3
	.finalH1H2 = Get mean: "h1h2"
endproc


# Perturbation
procedure getPerturbation: .sndid, .floor, .ceiling
	selectObject: .sndid
	.pitch = To Pitch (cc): 0, .floor, 15, "no", 0.03, 0.45, 0.01, 0.35, 0.14, .ceiling
	selectObject: .sndid, .pitch
	.pproc = To PointProcess (cc)
	selectObject: .sndid, .pitch, .pproc
	.voiceReport$ = Voice report: 0, 0, .floor, .ceiling, 1.3, 1.6, 0.03, 0.45
	.jitterLocalRaw = extractNumber (.voiceReport$, "Jitter (local):")
	.jitterLocal = .jitterLocalRaw * 100
	.jitterPPQ5Raw = extractNumber (.voiceReport$, "Jitter (ppq5):")
	.jitterPPQ5 = .jitterPPQ5Raw * 100
	.shimmerLocalRaw = extractNumber (.voiceReport$, "Shimmer (local):")
	.shimmerLocal = .shimmerLocalRaw * 100
	.shimmerLocaldB = extractNumber (.voiceReport$, "Shimmer (local, dB):")
	.hnr = extractNumber (.voiceReport$, "Mean harmonics-to-noise ratio:")
	.voiceBreaksRaw = extractNumber (.voiceReport$, "Degree of voice breaks:")
	.voiceBreaks = .voiceBreaksRaw * 100
	.psds = extractNumber (.voiceReport$, "Standard deviation of period:")
	.psd = .psds * 1000
endproc


# Glottal-to-Noise Excitation ratio (gne)
procedure getGNE: .sndid
	selectObject: .sndid
	.harm = To Harmonicity (gne): 500, 4500, 1000, 80
	.gneMax = Get maximum
endproc


# pitch
procedure getPitch: .sndid, .floor, .ceiling, .type$
	selectObject: .sndid
	.pitch = To Pitch (ac): 0, .floor, 15, "no", 0.03, 0.45, 0.01, 0.35, 0.14, .ceiling
	.minimumPitch = Get minimum: 0, 0, "Hertz", "parabolic"
	.maximumPitch = Get maximum: 0, 0, "Hertz", "parabolic"
	.meanPitch = Get mean: 0, 0, "Hertz"
	.sdPitch = Get standard deviation: 0, 0, "Hertz"
	.semitones = (12/ln(2)) * ln(.maximumPitch/.minimumPitch)
	selectObject: .pitch
	To Matrix
	if .type$ = "sv"
		Save as text file: sv_pitch_file$
	elsif .type$ = "cs"
		Save as text file: cs_pitch_file$
	endif
endproc





### PREPARE SAMPLES
if passage = 0
	sv_floor = undef_value
	sv_ceiling = undef_value
	cs_floor = undef_value
	cs_ceiling = undef_value
	mx_floor = undef_value
	mx_ceiling = undef_value
	resampled_cs = 0
	resampled_sv = 0

	### cs
	if samples$ = "cs" or samples$ = "mx"
		cs_raw = Read from file: cs_file$
		cs_filtered = Filter (stop Hann band): 0, 34, 0.1
		cs_orig_dur = Get total duration
		@getVoicedSegments: cs_filtered
		cs_sound = getVoicedSegments.result
		selectObject: cs_sound
		Save as text file: cs_analyzed_file$
		cs_voiced_dur = Get total duration
		@analysisRange: cs_raw, 2.5
		cs_floor = analysisRange.floor
		cs_ceiling = analysisRange.ceiling
	endif

	### sv
	if samples$ = "sv" or samples$ = "mx"
		sv_sound = Read from file: sv_file$
		Save as text file: sv_analyzed_file$
		@analysisRange: sv_sound, 1.5
		sv_floor = analysisRange.floor
		sv_ceiling = analysisRange.ceiling
	endif

	### sv and cs mixed
	if samples$ = "mx" and indices
		selectObject: cs_sound
		cssf = Get sampling frequency
		selectObject: sv_sound
		svsf = Get sampling frequency
		if cssf > svsf
			selectObject: cs_sound
			resampled_cs = Resample: svsf, 50
			selectObject: resampled_cs, sv_sound
			mx_sound = Concatenate
		elsif cssf < svsf
			selectObject: sv_sound
			resampled_sv = Resample: cssf, 50
			selectObject: cs_sound, resampled_sv
			mx_sound = Concatenate
		else
			selectObject: cs_sound, sv_sound
			mx_sound = Concatenate
		endif
		Save as text file: mx_analyzed_file$
		Save as WAV file: mx_file$
		@analysisRange: mx_sound, 2.5
		mx_floor = analysisRange.floor
		mx_ceiling = analysisRange.ceiling
	endif

	writeFileLine: praat_tmp_file$, "sv_floor: " + string$(sv_floor)
	appendFileLine: praat_tmp_file$, "sv_ceiling: " + string$(sv_ceiling)
	appendFileLine: praat_tmp_file$, "cs_floor: " + string$(cs_floor)
	appendFileLine: praat_tmp_file$, "cs_ceiling: " + string$(cs_ceiling)
	appendFileLine: praat_tmp_file$, "mx_floor: " + string$(mx_floor)
	appendFileLine: praat_tmp_file$, "mx_ceiling: " + string$(mx_ceiling)
	if resampled_cs > 0
		appendFileLine: praat_tmp_file$, "resampled: " + "1"
	elsif resampled_sv > 0
		appendFileLine: praat_tmp_file$, "resampled: " + "2"
	else
		appendFileLine: praat_tmp_file$, "resampled: " + "0"
	endif
	exitScript()


### ANALYSES: Slope & Tilt
elsif passage = 1
	sv_slope = undef_value
	sv_tilt = undef_value
	cs_slope = undef_value
	cs_tilt = undef_value
	mx_slope = undef_value
	mx_tilt = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		@getSlope: cs_sound
		cs_slope = getSlope.slope
		cs_tilt = getSlope.tilt
	endif
	if samples$ = "sv" or samples$ = "mx"
		@getSlope: sv_sound
		sv_slope = getSlope.slope
		sv_tilt = getSlope.tilt
	endif
	if samples$ = "mx" and indices
		@getSlope: mx_sound
		mx_slope = getSlope.slope
		mx_tilt = getSlope.tilt
	endif
	appendFileLine: praat_tmp_file$, "sv_slope: " + string$(sv_slope)
	appendFileLine: praat_tmp_file$, "sv_tilt: " + string$(sv_tilt)
	appendFileLine: praat_tmp_file$, "cs_slope: " + string$(cs_slope)
	appendFileLine: praat_tmp_file$, "cs_tilt: " + string$(cs_tilt)
	appendFileLine: praat_tmp_file$, "mx_slope: " + string$(mx_slope)
	appendFileLine: praat_tmp_file$, "mx_tilt: " + string$(mx_tilt)
	exitScript()


### ANALYSES: HFNo, HNRd & H1H2
elsif passage = 2
	sv_hfno = undef_value
	sv_hnrd = undef_value
	sv_h1h2 = undef_value
	cs_hfno = undef_value
	cs_hnrd = undef_value
	cs_h1h2 = undef_value
	mx_hfno = undef_value
	mx_hnrd = undef_value
	mx_h1h2 = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		@getABIparms: cs_sound
		cs_hfno = getABIparms.finalhfno6000
		cs_hnrd = getABIparms.finalhnrd
		cs_h1h2 = getABIparms.finalH1H2
	endif
	if samples$ = "sv" or samples$ = "mx"
		@getABIparms: sv_sound
		sv_hfno = getABIparms.finalhfno6000
		sv_hnrd = getABIparms.finalhnrd
		sv_h1h2 = getABIparms.finalH1H2
	endif
	if samples$ = "mx" and indices
		@getABIparms: mx_sound
		mx_hfno = getABIparms.finalhfno6000
		mx_hnrd = getABIparms.finalhnrd
		mx_h1h2 = getABIparms.finalH1H2
	endif
	appendFileLine: praat_tmp_file$, "sv_hfno: " + string$(sv_hfno)
	appendFileLine: praat_tmp_file$, "sv_hnrd: " + string$(sv_hnrd)
	appendFileLine: praat_tmp_file$, "sv_h1h2: " + string$(sv_h1h2)
	appendFileLine: praat_tmp_file$, "cs_hfno: " + string$(cs_hfno)
	appendFileLine: praat_tmp_file$, "cs_hnrd: " + string$(cs_hnrd)
	appendFileLine: praat_tmp_file$, "cs_h1h2: " + string$(cs_h1h2)
	appendFileLine: praat_tmp_file$, "mx_hfno: " + string$(mx_hfno)
	appendFileLine: praat_tmp_file$, "mx_hnrd: " + string$(mx_hnrd)
	appendFileLine: praat_tmp_file$, "mx_h1h2: " + string$(mx_h1h2)
	exitScript()


### ANALYSES: CPPS
elsif passage = 3
	sv_cpps = undef_value
	@passageRead
	if samples$ = "sv" or samples$ = "mx"
		@getCPPS: sv_sound
		sv_cpps = getCPPS.result
	endif
	appendFileLine: praat_tmp_file$, "sv_cpps: " + string$(sv_cpps)
	exitScript()
elsif passage = 4
	cs_cpps = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		@getCPPS: cs_sound
		cs_cpps = getCPPS.result
	endif
	appendFileLine: praat_tmp_file$, "cs_cpps: " + string$(cs_cpps)
	exitScript()
elsif passage = 5
	mx_cpps = undef_value
	@passageRead
	if samples$ = "mx" and indices
		@getCPPS: mx_sound
		mx_cpps = getCPPS.result
	endif
	appendFileLine: praat_tmp_file$, "mx_cpps: " + string$(mx_cpps)
	exitScript()


### ANALYSES: Perturbation
elsif passage = 6
	sv_jitterLocal = undef_value
	sv_jitterPPQ5 = undef_value
	sv_shimmerLocal = undef_value
	sv_shimmerLocaldB = undef_value
	sv_psd = undef_value
	sv_hnr = undef_value
	sv_voiceBreaks = undef_value
	cs_jitterLocal = undef_value
	cs_jitterPPQ5 = undef_value
	cs_shimmerLocal = undef_value
	cs_shimmerLocaldB = undef_value
	cs_psd = undef_value
	cs_hnr = undef_value
	mx_jitterLocal = undef_value
	mx_jitterPPQ5 = undef_value
	mx_shimmerLocal = undef_value
	mx_shimmerLocaldB = undef_value
	mx_psd = undef_value
	mx_hnr = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		@getPerturbation: cs_sound, cs_floor, cs_ceiling
		cs_jitterLocal = getPerturbation.jitterLocal
		cs_jitterPPQ5 = getPerturbation.jitterPPQ5
		cs_shimmerLocal = getPerturbation.shimmerLocal
		cs_shimmerLocaldB = getPerturbation.shimmerLocaldB
		cs_psd = getPerturbation.psd
		cs_hnr = getPerturbation.hnr
	endif
	if samples$ = "sv" or samples$ = "mx"
		@getPerturbation: sv_sound, sv_floor, sv_ceiling
		sv_jitterLocal = getPerturbation.jitterLocal
		sv_jitterPPQ5 = getPerturbation.jitterPPQ5
		sv_shimmerLocal = getPerturbation.shimmerLocal
		sv_shimmerLocaldB = getPerturbation.shimmerLocaldB
		sv_psd = getPerturbation.psd
		sv_hnr = getPerturbation.hnr
		sv_voiceBreaks = getPerturbation.voiceBreaks
	endif
	if samples$ = "mx" and indices
		@getPerturbation: mx_sound, mx_floor, mx_ceiling
		mx_jitterLocal = getPerturbation.jitterLocal
		mx_jitterPPQ5 = getPerturbation.jitterPPQ5
		mx_shimmerLocal = getPerturbation.shimmerLocal
		mx_shimmerLocaldB = getPerturbation.shimmerLocaldB
		mx_psd = getPerturbation.psd
		mx_hnr = getPerturbation.hnr
	endif
	appendFileLine: praat_tmp_file$, "sv_jitterLocal: " + string$(sv_jitterLocal)
	appendFileLine: praat_tmp_file$, "sv_jitterPPQ5: " + string$(sv_jitterPPQ5)
	appendFileLine: praat_tmp_file$, "sv_shimmerLocal: " + string$(sv_shimmerLocal)
	appendFileLine: praat_tmp_file$, "sv_shimmerLocaldB: " + string$(sv_shimmerLocaldB)
	appendFileLine: praat_tmp_file$, "sv_psd: " + string$(sv_psd)
	appendFileLine: praat_tmp_file$, "sv_hnr: " + string$(sv_hnr)
	appendFileLine: praat_tmp_file$, "sv_voiceBreaks: " + string$(sv_voiceBreaks)
	appendFileLine: praat_tmp_file$, "cs_jitterLocal: " + string$(cs_jitterLocal)
	appendFileLine: praat_tmp_file$, "cs_jitterPPQ5: " + string$(cs_jitterPPQ5)
	appendFileLine: praat_tmp_file$, "cs_shimmerLocal: " + string$(cs_shimmerLocal)
	appendFileLine: praat_tmp_file$, "cs_shimmerLocaldB: " + string$(cs_shimmerLocaldB)
	appendFileLine: praat_tmp_file$, "cs_psd: " + string$(cs_psd)
	appendFileLine: praat_tmp_file$, "cs_hnr: " + string$(cs_hnr)
	appendFileLine: praat_tmp_file$, "mx_jitterLocal: " + string$(mx_jitterLocal)
	appendFileLine: praat_tmp_file$, "mx_jitterPPQ5: " + string$(mx_jitterPPQ5)
	appendFileLine: praat_tmp_file$, "mx_shimmerLocal: " + string$(mx_shimmerLocal)
	appendFileLine: praat_tmp_file$, "mx_shimmerLocaldB: " + string$(mx_shimmerLocaldB)
	appendFileLine: praat_tmp_file$, "mx_psd: " + string$(mx_psd)
	appendFileLine: praat_tmp_file$, "mx_hnr: " + string$(mx_hnr)
	exitScript()


### ANALYSES: GNE
elsif passage = 7
	sv_gne = undef_value
	@passageRead
	if samples$ = "sv" or samples$ = "mx"
		@getGNE: sv_sound
		sv_gne = getGNE.gneMax
	endif
	appendFileLine: praat_tmp_file$, "sv_gne: " + string$(sv_gne)
	exitScript()
elsif passage = 8
	cs_gne = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		@getGNE: cs_sound
		cs_gne = getGNE.gneMax
	endif
	appendFileLine: praat_tmp_file$, "cs_gne: " + string$(cs_gne)
	exitScript()
elsif passage = 9
	mx_gne = undef_value
	@passageRead
	if samples$ = "mx" and indices
		@getGNE: mx_sound
		mx_gne = getGNE.gneMax
	endif
	appendFileLine: praat_tmp_file$, "mx_gne: " + string$(mx_gne)
	exitScript()


### ANALYSES: Pitch
elsif passage = 10
	sv_sdPitch = undef_value
	sv_meanPitch = undef_value
	sv_minimumPitch = undef_value
	sv_maximumPitch = undef_value
	sv_semitones = undef_value
	cs_sdPitch = undef_value
	cs_meanPitch = undef_value
	cs_minimumPitch = undef_value
	cs_maximumPitch = undef_value
	cs_semitones = undef_value
	@passageRead
	if samples$ = "cs" or samples$ = "mx"
		file = Read from file: cs_file$
		@getPitch: file, cs_floor, cs_ceiling, "cs"
		cs_sdPitch = getPitch.sdPitch
		cs_meanPitch = getPitch.meanPitch
		cs_minimumPitch = getPitch.minimumPitch
		cs_maximumPitch = getPitch.maximumPitch
		cs_semitones = getPitch.semitones
	endif
	if samples$ = "sv" or samples$ = "mx"
		file = Read from file: sv_file$
		@getPitch: file, sv_floor, sv_ceiling, "sv"
		sv_sdPitch = getPitch.sdPitch
		sv_meanPitch = getPitch.meanPitch
		sv_minimumPitch = getPitch.minimumPitch
		sv_maximumPitch = getPitch.maximumPitch
		sv_semitones = getPitch.semitones
	endif
	appendFileLine: praat_tmp_file$, "sv_sdPitch: " + string$(sv_sdPitch)
	appendFileLine: praat_tmp_file$, "sv_meanPitch: " + string$(sv_meanPitch)
	appendFileLine: praat_tmp_file$, "sv_minimumPitch: " + string$(sv_minimumPitch)
	appendFileLine: praat_tmp_file$, "sv_maximumPitch: " + string$(sv_maximumPitch)
	appendFileLine: praat_tmp_file$, "sv_semitones: " + string$(sv_semitones)
	appendFileLine: praat_tmp_file$, "cs_sdPitch: " + string$(cs_sdPitch)
	appendFileLine: praat_tmp_file$, "cs_meanPitch: " + string$(cs_meanPitch)
	appendFileLine: praat_tmp_file$, "cs_minimumPitch: " + string$(cs_minimumPitch)
	appendFileLine: praat_tmp_file$, "cs_maximumPitch: " + string$(cs_maximumPitch)
	appendFileLine: praat_tmp_file$, "cs_semitones: " + string$(cs_semitones)
	exitScript()


### ANALYSES: Indices
elsif passage = 11
	avqi = undef_value
	abi = undef_value
	@passageRead
	if samples$ = "mx" and indices
		# AVQI v03.01 (Barsties & Maryn, 2015;  Barsties et al., 2018/in press)
		avqi = (4.152 - (0.177*mx_cpps) - (0.006*mx_hnr) - (0.037*mx_shimmerLocal) + (0.941*mx_shimmerLocaldB) + (0.01*mx_slope) + (0.093*mx_tilt)) * 2.8902
		# ABI (Barsties et al., 2017)
		abi =(5.0447730915-(0.172*mx_cpps)-(0.193*mx_jitterLocal)-(1.283*mx_gne)-(0.396*mx_hfno)+(0.01*mx_hnrd)+(0.017*mx_h1h2)+(1.473*mx_shimmerLocaldB)-(0.088*mx_shimmerLocal)-(68.295*(mx_psd/1000)))*2.9257400394
	endif
	appendFileLine: praat_tmp_file$, "avqi: " + string$(avqi)
	appendFileLine: praat_tmp_file$, "abi: " + string$(abi)
	exitScript()
endif



