# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import os
import pickle
import pyaudio



def fs_access_test(app_dir, working_dir, tmp_dir):
	if not os.access(app_dir, os.R_OK):
		return "Can't access app dir at " + app_dir

	if not os.access(working_dir, os.F_OK):
		try:
			os.mkdir(working_dir)
		except Exception as e:
			return "Can't create working dir at " + working_dir + "\n\n" + e.args[0]

	elif not os.access(working_dir, os.W_OK):
		return "Read/write access to working dir was refused: " + working_dir

	if not os.access(tmp_dir, os.F_OK):
		try:
			os.mkdir(tmp_dir)
		except Exception as e:
			return "Can't create tmp dir at " + tmp_dir + "\n\n" + e.args[0]

	elif not os.access(tmp_dir, os.W_OK):
		return "Read/write access to tmp dir was refused: " + tmp_dir

	return False



def read_settings(settings_file, USet):
	'''Read user settings if available

	:return: Boolean, settings available (False) or not (True)
	'''
	if os.access(settings_file, os.F_OK | os.R_OK) and os.stat(settings_file).st_size > 0:
		settings_data = pickle.load(open(settings_file, 'rb'))
		if 'rdcs' in settings_data:
			USet.rec_duration['cs'] = settings_data['rdcs']
		if 'rdsv' in settings_data:
			USet.rec_duration['sv'] = settings_data['rdsv']
		if 'ex' in settings_data:
			USet.examiner = settings_data['ex']
		if 'ilang' in settings_data:
			USet.i_lang = settings_data['ilang']
		if 'alang' in settings_data:
			USet.a_lang = settings_data['alang']
		if 'csvf' in settings_data:
			USet.csv_field_delim = settings_data['csvf']
		if 'csvd' in settings_data:
			USet.csv_decimal_sep = settings_data['csvd']
		if 'datef' in settings_data:
			USet.date_format = settings_data['datef']
		if 'timef' in settings_data:
			USet.time_format = settings_data['timef']
		if 'lplot' in settings_data:
			USet.live_plot = settings_data['lplot']
		return False
	else:
		return True



def set_gui_language(lcodes, translations, USet):
	'''GUI language settings

	:param lcodes: List of strings, codes of available languages
	:param translations: Dict of available translations
	:param Uset: Object, user settings

	:return translations[c].gettext: Object, user selected translation
	'''
	import gettext
	c = lcodes[USet.i_lang]
	translations[c].install
	return translations[c].gettext



def set_ana_language(alang_data, a_lang):
	'''Analysis language settings

	:param alang_data: List, complete list of available analysis language data
	:param a_lang: Int, index of user selected analysis language

	:return analysis_langs: List of strings, all available analysis languages
	:return cs_text: String, reading text of target analysis language for continuous speech sample (actual text or png image file name)
	:return norm: Dict, norm values of target analysis language
	:return maxv: Dict, maximum values of acoustic parameters
	'''
	# generate list of available languages
	analysis_langs = []
	for data_record in alang_data:
		analysis_langs.append(data_record['language'] + ' (' + data_record['language_code'] + ')')

	# get reading text of selected language
	cs_text = alang_data[a_lang]['reading_text'].replace('\\n', '\n')

	# set norm values for selected language
	norm = {}
	norm['slope'] = (None if alang_data[a_lang]['slope'] == 'None' else float(alang_data[a_lang]['slope'].replace(',', '.')))
	norm['tilt'] = (None if alang_data[a_lang]['tilt'] == 'None' else float(alang_data[a_lang]['tilt'].replace(',', '.')))
	norm['hfno'] = (None if alang_data[a_lang]['hfno'] == 'None' else float(alang_data[a_lang]['hfno'].replace(',', '.')))
	norm['hnrd'] = (None if alang_data[a_lang]['hnrd'] == 'None' else float(alang_data[a_lang]['hnrd'].replace(',', '.')))
	norm['h1h2'] = (None if alang_data[a_lang]['h1h2'] == 'None' else float(alang_data[a_lang]['h1h2'].replace(',', '.')))
	norm['cpps'] = (None if alang_data[a_lang]['cpps'] == 'None' else float(alang_data[a_lang]['cpps'].replace(',', '.')))
	norm['jitterLocal'] = (None if alang_data[a_lang]['jitterLocal'] == 'None' else float(alang_data[a_lang]['jitterLocal'].replace(',', '.')))
	norm['jitterPPQ5'] = (None if alang_data[a_lang]['jitterPPQ5'] == 'None' else float(alang_data[a_lang]['jitterPPQ5'].replace(',', '.')))
	norm['shimmerLocal'] = (None if alang_data[a_lang]['shimmerLocal'] == 'None' else float(alang_data[a_lang]['shimmerLocal'].replace(',', '.')))
	norm['shimmerLocaldB'] = (None if alang_data[a_lang]['shimmerLocaldB'] == 'None' else float(alang_data[a_lang]['shimmerLocaldB'].replace(',', '.')))
	norm['hnr'] = (None if alang_data[a_lang]['hnr'] == 'None' else float(alang_data[a_lang]['hnr'].replace(',', '.')))
	norm['psd'] = (None if alang_data[a_lang]['psd'] == 'None' else float(alang_data[a_lang]['psd'].replace(',', '.')))
	norm['voiceBreaks'] = (None if alang_data[a_lang]['voiceBreaks'] == 'None' else float(alang_data[a_lang]['voiceBreaks'].replace(',', '.')))
	norm['gne'] = (None if alang_data[a_lang]['gne'] == 'None' else float(alang_data[a_lang]['gne'].replace(',', '.')))
	norm['meanPitch'] = (None if alang_data[a_lang]['meanPitch'] == 'None' else float(alang_data[a_lang]['meanPitch'].replace(',', '.')))
	norm['minimumPitch'] = (None if alang_data[a_lang]['minimumPitch'] == 'None' else float(alang_data[a_lang]['minimumPitch'].replace(',', '.')))
	norm['maximumPitch'] = (None if alang_data[a_lang]['maximumPitch'] == 'None' else float(alang_data[a_lang]['maximumPitch'].replace(',', '.')))
	norm['sdPitch'] = (None if alang_data[a_lang]['sdPitch'] == 'None' else float(alang_data[a_lang]['sdPitch'].replace(',', '.')))
	norm['semitones'] = (None if alang_data[a_lang]['semitones'] == 'None' else float(alang_data[a_lang]['semitones'].replace(',', '.')))
	norm['avqi'] = (None if alang_data[a_lang]['avqi'] == 'None' else float(alang_data[a_lang]['avqi'].replace(',', '.')))
	norm['abi'] = (None if alang_data[a_lang]['abi'] == 'None' else float(alang_data[a_lang]['abi'].replace(',', '.')))

	# set norm value strings for selected language
	norm['slopex'] = ('{:.2f}'.format(norm['slope']) if norm['slope'] else ' ')
	norm['tiltx'] = ('{:.2f}'.format(norm['tilt']) if norm['tilt'] else ' ')
	norm['hfnox'] = ('> ' + '{:.2f}'.format(norm['hfno']) if norm['hfno'] else ' ')
	norm['hnrdx'] = ('> ' + '{:.2f}'.format(norm['hnrd']) if norm['hnrd'] else ' ')
	norm['h1h2x'] = ('{:.2f}'.format(norm['h1h2']) if norm['h1h2'] else ' ')
	norm['cppsx'] = ('> ' + '{:.2f}'.format(norm['cpps']) if norm['cpps'] else ' ')
	norm['jitterLocalx'] = ('< ' + '{:.2f}'.format(norm['jitterLocal']) if norm['jitterLocal'] else ' ')
	norm['jitterPPQ5x'] = ('< ' + '{:.2f}'.format(norm['jitterPPQ5']) if norm['jitterPPQ5'] else ' ')
	norm['shimmerLocalx'] = ('< ' + '{:.2f}'.format(norm['shimmerLocal']) if norm['shimmerLocal'] else ' ')
	norm['shimmerLocaldBx'] = ('< ' + '{:.2f}'.format(norm['shimmerLocaldB']) if norm['shimmerLocaldB'] else ' ')
	norm['hnrx'] = ('> ' + '{:.2f}'.format(norm['hnr']) if norm['hnr'] else ' ')
	norm['psdx'] = ('< ' + '{:.2f}'.format(norm['psd']) if norm['psd'] else ' ')
	norm['voiceBreaksx'] = ('< ' + '{:.2f}'.format(norm['voiceBreaks']) if norm['voiceBreaks'] else ' ')
	norm['gnex'] = ('> ' + '{:.2f}'.format(norm['gne']) if norm['gne'] else ' ')
	norm['meanPitchx'] = ('{:.2f}'.format(norm['meanPitch']) if norm['meanPitch'] else ' ')
	norm['minimumPitchx'] = ('{:.2f}'.format(norm['minimumPitch']) if norm['minimumPitch'] else ' ')
	norm['maximumPitchx'] = ('{:.2f}'.format(norm['maximumPitch']) if norm['maximumPitch'] else ' ')
	norm['sdPitchx'] = ('{:.2f}'.format(norm['sdPitch']) if norm['sdPitch'] else ' ')
	norm['semitonesx'] = ('{:.2f}'.format(norm['semitones']) if norm['semitones'] else ' ')
	norm['avqix'] = ('< ' + '{:.2f}'.format(norm['avqi']) if norm['avqi'] else ' ')
	norm['abix'] = ('< ' + '{:.2f}'.format(norm['abi']) if norm['abi'] else ' ')

	return analysis_langs, cs_text, norm



def praat_test(praat_exe, test_script, praat_tmp_file, praat_version):
	import subprocess

	if not os.access(praat_exe, os.X_OK):
		return "Can't find executable Praat application at:" + '\n\n' + praat_exe

	try:
		subprocess.run([praat_exe, '--run', '--no-pref-files', '--no-plugins', '--utf8', test_script, praat_tmp_file], check=True)
	except subprocess.CalledProcessError as e:
		return "A fatal error occurred during Praat version test."

	ptf = open(praat_tmp_file, 'r')
	found_praat_version = ptf.readline().strip()
	ptf.close()
	if not found_praat_version == praat_version:
		return "Found Praat version:" + '\n\n' + praat_version + '\n\n' + "Required Praat version:" + '\n\n' + praat_version

	return False



def input_device_test(ASet):
	_ = ASet.translate
	p = pyaudio.PyAudio()
	api_info = p.get_host_api_info_by_index(0)
	num_devices = api_info.get('deviceCount')
	devices = {}
	for i in range (0, num_devices):
		device = p.get_device_info_by_host_api_device_index(0,i)
		if device.get('maxInputChannels') > 0:
			devices.update({str(i) : device.get('name')})

	if not devices:
		input_device_available = False
		device_info = {
			'index' : '',
			'name' : _("No input device (microphone) found."),
			'maxInputChannels' : '',
			'defaultSampleRate' : ''
			}
	else:
		input_device_available = True
		default_input = p.get_default_input_device_info()
		di_index = default_input.get('index')
		di_name = (devices[str(di_index)] if str(di_index) in devices else 'Default Input')
		di_channels = default_input.get('maxInputChannels')
		dsr = int(default_input.get('defaultSampleRate'))
		ASet.default_sampling_rate = dsr
		ASet.sampling_rate = dsr
		ASet.v_frames_analyzed = ASet.v_seconds_analyzed * dsr
		device_info = {
			'index' : di_index,
			'name' : di_name,
			'maxInputChannels' : str(di_channels),
			'defaultSampleRate' : str(dsr)
			}
	p.terminate()
	return input_device_available, device_info



def input_devices_debug():
	devices_info = ""
	p = pyaudio.PyAudio()
	api_info = p.get_host_api_info_by_index(0)
	num_devices = api_info.get('deviceCount')
	for i in range (0, num_devices):
		device = p.get_device_info_by_host_api_device_index(0,i)
		if device.get('maxInputChannels') > 0:
			try:
				p.is_format_supported(44100.0, input_device=i, input_channels=2, input_format=pyaudio.paInt16)
			except:
				supported = 'False'
			else:
				supported = 'True'

			devices_info = (
				devices_info + '\n' + str(i) + ' ' + device.get('name') + '\n'
				'maxInputChannels: ' + str(device.get('maxInputChannels')) + '\n'
				'maxOutputChannels: ' + str(device.get('maxOutputChannels')) + '\n'
				'defaultSampleRate: ' + str(device.get('defaultSampleRate')) + '\n'
				'44,1 kHz supported: ' + supported + "\n"
				)
	devices_info = devices_info + "\ndefault input: " + str(p.get_default_input_device_info())
	p.terminate()
	return devices_info



def toggle_root_state(root, state='normal'):
	widget_list = root.winfo_children()
	# recursively find all widgets
	for widget in widget_list:
		if widget.winfo_children():
			widget_list.extend(widget.winfo_children())
	# toggle all widget's state (if they have state)
	for widget in widget_list:
		try:
			widget['state'] = state
		except:
			pass



