# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import sys, os, platform
import tkinter as tk
from tkinter import ttk, font
from tkinter.messagebox import showerror
import base64
from voxplot import helpers, app



class AppSettings:
	'''Manage constants and global variables

	Initialize variables and assign constant/default values
	
	:param root: Object, reference to TK root window
	'''
	def __init__(self, root):
		self.app_name = 'VOXplot'
		self.os_type = platform.system()

		# directories
		self.home_dir = os.path.expanduser("~")
		self.app_dir, self.is_bundled = self._get_appdir()
		self.working_dir, self.praat_exe, self.font_size = self._get_os_dependent()
		self.tmp_dir = os.path.join(self.working_dir, 'tmp')
		self.data_dir = os.path.join(self.app_dir, 'data')

		# persistent files
		self.settings_file = os.path.join(self.working_dir, 'settings.dat')
		self.test_script = os.path.join(self.data_dir, 'praat_test.praat')
		self.conv_script = os.path.join(self.data_dir, 'praat_convert.praat')
		self.praat_script = os.path.join(self.data_dir, 'praat_analysis.praat')
		self.logo_xs = os.path.join(self.data_dir, 'logo_xs.png')
		self.logo_s = os.path.join(self.data_dir, 'logo_s.png')
		self.logo_b64 = self._get_b64enc(self.logo_xs)
		self.app_version, self.app_date = self._get_version_info()

		# temporary files
		self.sv_file = os.path.join(self.tmp_dir, 'sv.wav')
		self.cs_file = os.path.join(self.tmp_dir, 'cs.wav')
		self.mx_file = os.path.join(self.tmp_dir, 'mx.wav')
		self.sv_analyzed_file = os.path.join(self.tmp_dir, 'sv_analyzed.txt')
		self.cs_analyzed_file = os.path.join(self.tmp_dir, 'cs_analyzed.txt')
		self.mx_analyzed_file = os.path.join(self.tmp_dir, 'mx_analyzed.txt')
		self.sv_pitch_file = os.path.join(self.tmp_dir, 'sv_pitch.txt')
		self.cs_pitch_file = os.path.join(self.tmp_dir, 'cs_pitch.txt')
		self.tmp_file = os.path.join(self.tmp_dir, 'tmp.wav')
		self.response_tmp_file = os.path.join(self.tmp_dir, 'tmp_response.wav')
		self.praat_tmp_file = os.path.join(self.tmp_dir, 'praat_tmp')
		self.conv_tmp_file = os.path.join(self.tmp_dir, 'conv_tmp.wav')
		self.pdf_file = os.path.join(self.tmp_dir, 'results.pdf')
		self.png_file = os.path.join(self.tmp_dir, 'results.png')
		self.jpg_file = os.path.join(self.tmp_dir, 'results.jpg')
		self.tmp_files = (
			self.sv_file,
			self.cs_file,
			self.mx_file,
			self.sv_analyzed_file,
			self.cs_analyzed_file,
			self.mx_analyzed_file,
			self.sv_pitch_file,
			self.cs_pitch_file,
			self.tmp_file,
			self.response_tmp_file,
			self.praat_tmp_file,
			self.conv_tmp_file,
			self.pdf_file,
			self.png_file,
			self.jpg_file,
			)

		# localization
		self.gui_langs = ('English', 'Deutsch', 'Nederlands')
		self.gui_langcodes = ('en', 'de', 'nl')
		self.translations = {}
		for l in self.gui_langcodes:
			self.translations[l] = self._get_translation(l)
		self.translate = None
		self.csv_field_delim_list = [' , (comma)', ' ; (semi-colon)', ' TAB (tab stop)']
		self.csv_decimal_sep_list = [' . (point)', ' , (comma)']
		self.date_format_list = ['mm/dd/yyyy', 'dd.mm.yyyy', 'yyyy-mm-dd']
		self.time_format_list = ['12h', '24h']

		# analysis languages
		self.alang_data = self._get_alang_data()
		self.norm_values = None
		self.analysis_langs = None
		self.cs_text = None

		# window instances
		self.root = root
		self.app_win = None
		self.rec_win = None
		self.snr_win = None

		# miscellaneous
		self.praat_version = '6048'
		self.no_settings = True
		self.input_device_available = False
		self.audio_info = None
		self.default_sampling_rate = 44100
		self.sampling_rate = self.default_sampling_rate
		self.chunk = 8192
		# self.chunk = 4096
		self.channels = 1
		self.v_seconds_analyzed = 3
		self.v_frames_analyzed = self.v_seconds_analyzed * self.sampling_rate
		self.sv_too_short = False
		self.sv_too_long = False
		self.screen_height = None
		self.style = None
		self.default_font = None
		self.font_family = 'Helvetica'
		# self.font_family = 'Times'
		# signal = self._get_signal()


	def _get_appdir(self):
		if getattr(sys, 'frozen', False):
			# we are running in a bundle (pyinstaller --onefile)
			is_bundled = True
			meipass_dir = sys._MEIPASS
			app_dir = os.path.join(meipass_dir, 'voxplot')
		else:
			# we are running in a normal Python environment
			is_bundled = False
			app_dir = os.path.dirname(os.path.abspath(__file__))
		return app_dir, is_bundled


	def _get_os_dependent(self):
		'''Misc OS dependent stuff

		:return working_dir: Str, path to working dir
		:return praat_exe: Str, path to Praat executable
		:return font_size: Int, global font size
		'''
		working_dir_name = 'voxplot-dir'
		if 'Linux' in self.os_type:
			working_dir = os.path.join(self.home_dir, '.' + working_dir_name)
			praat_exe = os.path.join(self.app_dir, 'Praat', 'praat')
			font_size = 10
		elif 'Darwin' in self.os_type:
			working_dir = os.path.join(self.home_dir, '.' + working_dir_name)
			praat_exe = os.path.join(self.app_dir, 'Praat', 'Praat.app/Contents/MacOS/Praat')
			font_size = 12
		else:
			locappdata = os.getenv('LOCALAPPDATA')
			working_dir = os.path.join(locappdata, working_dir_name)
			praat_exe = os.path.join(self.app_dir, 'Praat', 'Praat.exe')
			font_size = 11
		return working_dir, praat_exe, font_size


	def _get_b64enc(self, logo):
		'''Convert logo to base64 (required for plattform independent window icon)
		'''
		with open(logo, 'rb') as img_file:
			logo_b64 = base64.b64encode(img_file.read())
		return logo_b64


	def _get_version_info(self):
		vfile = os.path.join(self.data_dir, 'version')
		v = open(vfile, 'r')
		app_version, app_date = v.readline().split(',')
		v.close()
		return app_version, app_date


	def _get_translation(self, lang_code):
		'''Initialize string translations
		
		:param lang_code: Str, code of target language

		:return	translation: Reference to language specific translation
		'''
		import gettext
		locales_dir = os.path.join(self.app_dir, 'locales')
		is_fallback = True if lang_code == 'en' else False
		translation = gettext.translation('voxplot', locales_dir, [lang_code], fallback=is_fallback)
		return translation


	def _get_alang_data(self):
		'''Initialize analysis languages

		Read from alang.csv (language names, reading texts, norm values)
		and convert to list.
		
		:return	alang_data: List, complete list of available analysis language data
		'''
		from csv import DictReader
		alang_file = os.path.join(self.data_dir, 'alang.csv')
		tmp_file = open(alang_file, encoding="utf8")
		tmp_data = DictReader(tmp_file)
		alang_data = list(tmp_data)
		tmp_file.close()
		return alang_data


	def _get_signal(self):
		from pydub import AudioSegment
		tone_wav = os.path.join(self.data_dir, 'tone.wav')
		signal = AudioSegment.from_wav(tone_wav)
		return signal



class UserSettings:
	'''Manage user settings

	Initialize variables and assign default values.
	'''
	def __init__(self):
		self.rec_duration = {}
		self.rec_duration['cs'] = 10
		self.rec_duration['sv'] = 6
		self.live_plot = True
		self.examiner = ''
		self.i_lang = 0
		self.a_lang = 1
		self.csv_field_delim = 0
		self.csv_decimal_sep = 0
		self.date_format = 0
		self.time_format = 0



def run():
	'''Run VOXplot
	
	Run basic tests/initializations and launch main loop
	'''
	root = tk.Tk()
	ASet = AppSettings(root)
	USet = UserSettings()
	ASet.screen_height = root.winfo_screenheight()

	### alternative method to determine screen height;
	### more accurate with multiple screen setups?
	# root.update_idletasks()
	# root.attributes('-fullscreen', True)
	# root.state('iconic')
	# tmp = root.winfo_geometry()
	# tmp = tmp.split('x')
	# tmp = tmp[1].split('+')
	# screen_height = tmp[0]
	# root.attributes('-fullscreen', False)
	# root.state('normal')

	ASet.default_font = font.nametofont("TkDefaultFont")
	ASet.default_font.config(family=ASet.font_family, size=ASet.font_size)

	root.iconphoto(True, tk.PhotoImage(data=ASet.logo_b64))
	root.title(ASet.app_name)

	ASet.style = ttk.Style(root)
	# built-in themes: clam, alt, default, classic
	ASet.style.theme_use('clam')

	error_msg = helpers.fs_access_test(
		ASet.app_dir,
		ASet.working_dir,
		ASet.tmp_dir
		)
	if error_msg:
		showerror("Error", error_msg)
		exit(1)
	
	ASet.no_settings = helpers.read_settings(ASet.settings_file, USet)
	ASet.translate = helpers.set_gui_language(ASet.gui_langcodes, ASet.translations, USet)
	ASet.analysis_langs, ASet.cs_text, ASet.norm_values = helpers.set_ana_language(ASet.alang_data, USet.a_lang)
	ASet.input_device_available, ASet.audio_info = helpers.input_device_test(ASet)

	error_msg = helpers.praat_test(
		ASet.praat_exe,
		ASet.test_script,
		ASet.praat_tmp_file,
		ASet.praat_version
		)
	if error_msg:
		showerror("Error", error_msg)
		exit(1)

	# close splash screen (only if bundled with pyinstaller)
	try:
		import pyi_splash
		if pyi_splash.is_alive():
			pyi_splash.close()
	except:
		pass

	# all tests passed, launch app
	ASet.app_win = app.Window(root, ASet, USet)
	root.mainloop()


