# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



import tkinter as tk
from tkinter import ttk
import numpy as np
import threading
import os
import time
import wave
import pyaudio
from pydub import AudioSegment
from voxplot import signalplot




class Window(tk.Toplevel):
	'''Handle recording and editing of voicesamples.

	:param parent: Object, reference to parent window
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	:param type: String, type of signal (cs/sv)
	:param is_edit: Boolean, new recording (False) or edit existing recording (True)
	'''
	def __init__(self, parent, ASet, USet, type, is_edit=False):
		super().__init__(parent)
		self.resizable(tk.FALSE,tk.FALSE)
		self.grab_set()
		### topmost=1 prevents messagebox display on top...!
		self.attributes('-topmost', 1)
		self.aset = ASet
		self.uset = USet
		_ = self.aset.translate
		self.type = type
		self.is_edit = is_edit
		if self.type == 'cs':
			title_add = _("(continuous speech)")
		elif self.type == 'sv':
			title_add = _("(sustained vowel)")
		if not self.is_edit: self.title(_("Record new sample") + " " + title_add)
		if self.is_edit: self.title(_("Edit sample") + " " + title_add)
		
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)

		### frames and widgets
		mainframe = ttk.Frame(self, padding=(10,10,10,10))
		mainframe.grid(column=0, row=0)
		self.columnconfigure(0, weight=1)
		self.rowconfigure(0, weight=1)
		self.rowconfigure(1, weight=1)
		self.rowconfigure(2, weight=0)
		self.rowconfigure(3, weight=0)
		self.rowconfigure(4, weight=0)

		# heading
		heading_frame = ttk.Frame(mainframe)
		heading_frame.grid(column=0, row=0, sticky='ns')
		if self.type == 'cs':
			if not self.is_edit: heading = _("Start recording and read this text:")
			if self.is_edit: heading = _("Select this text:")
		elif self.type == 'sv':
			if not self.is_edit: heading = _("Start recording and sustain vowel /a/ for at least 3 seconds.")
			if self.is_edit: heading = _("Select 3 seconds of stable section of the vowel.")
		self.heading_label = ttk.Label(
			heading_frame,
			text=heading,
			font=f'-family {self.aset.font_family} -size 14 -weight bold',
			)
		self.heading_label.grid(column=0, row=0, pady=(0,20))

		# reading
		read_frame = ttk.Frame(mainframe)
		read_frame.grid(column=0, row=1, sticky='ns')
		if self.type == 'cs':
			if self.aset.cs_text.rfind("png") >= 0:
				# cs_text is transparent png image (h = 140 px, w < 650px)
				self.cs_text_img = tk.PhotoImage(file=os.path.join(self.aset.data_dir, self.aset.cs_text))
				self.read_label = ttk.Label(
					read_frame,
					image=self.cs_text_img,
					).grid(column=0, row=0)
			else:
				# cs_text is text
				self.read_label = ttk.Label(
					read_frame,
					text=self.aset.cs_text,
					font='-family Times -size 18 -slant italic'
					).grid(column=0, row=0)
		elif self.type == 'sv':
			self.read_label = ttk.Label(
				read_frame,
				text="\n...aaaaa...\n",
				font='-family Times -size 18 -slant italic'
				).grid(column=0, row=0)

		# device info
		rec_dev = (" " if self.is_edit else _("Recording device") + ': ' + self.aset.audio_info['name'] + ' (' + str(self.aset.sampling_rate) + ' Hz)')
		self.dev_info_label = ttk.Label(
			read_frame,
			text=rec_dev,
			font=f'-family {self.aset.font_family} -size {str(self.aset.font_size - 1)}',
			)
		self.dev_info_label.grid(column=0, row=1, pady=(30,0))

		# plot
		self.plot_box = ttk.Frame(mainframe)
		self.plot_box.grid(column=0, row=2, pady=0)

		# instructions
		instr_frame = ttk.Frame(mainframe)
		instr_frame.grid(column=0, row=3)
		self.instr_text = tk.Text(
			instr_frame,
			width=95,
			height=9,
			font='TkDefaultFont',
			relief='solid',
			padx=10,
			pady=10,
			wrap='word',
			background='#ccc'
			)
		self.instr_text.grid(column=0, row=1, pady=20, sticky='nswe')
		self.instr_text.tag_configure(
			'btn',
			background='#666',
			foreground='#fff',
			borderwidth=0,
			font=f'-family (self.aset.font_family -size {str(self.aset.font_size)} -weight bold',
			relief='solid'
			)
		self.instr_text.tag_configure(
			'key',
			background='#ccc',
			foreground='#666',
			borderwidth=1,
			font=f'-family (self.aset.font_family -size {str(self.aset.font_size)} -weight bold',
			relief='solid'
			)
		self.instr_text.tag_configure(
			'warn',
			foreground='red',
			borderwidth=0,
			font=f'-family (self.aset.font_family -size {str(self.aset.font_size)} -weight bold',
			relief='flat'
			)
		self.instr_text.tag_configure('center', justify='center')
		self.instr_text.configure(state='disabled')
		
		# buttons
		button_frame = ttk.Frame(mainframe)
		button_frame.grid(column=0, row=4, sticky='nswe')
		button_frame.columnconfigure(0, weight=0)
		button_frame.columnconfigure(1, weight=0)
		button_frame.columnconfigure(2, weight=0)
		button_frame.columnconfigure(3, weight=0)
		button_frame.columnconfigure(4, weight=0)
		button_frame.columnconfigure(5, weight=1)
		button_frame.rowconfigure(0, weight=1)
		self.toogle_rec_btn = ttk.Button(button_frame,
			text=_("Record"),
			command=self.start_recording
			)
		self.toogle_rec_btn.grid(column=0, row=0, sticky='w', padx=(0,5))
		self.toogle_playA_btn = ttk.Button(
			button_frame,
			text=_("Play All"),
			command=lambda: self.start_playing(playall=True)
			)
		self.toogle_playA_btn.grid(column=1, row=0, sticky='w', padx=(0,5))
		self.toogle_playS_btn = ttk.Button(
			button_frame,
			text=_("Play Selection"),
			command=lambda: self.start_playing(playall=False)
			)
		self.toogle_playS_btn.grid(column=2, row=0, sticky='w', padx=(0,5))

		self.rm_save_btn = ttk.Button(
			button_frame,
			text=_("Accept"),
			command=self.save_pressed
			)
		self.rm_save_btn.grid(column=4, row=0, sticky='w', padx=(0,5))
		self.rm_cancel_btn = ttk.Button(
			button_frame,
			text=_("Cancel"),
			command=self.cancel_pressed
			)
		self.rm_cancel_btn.grid(column=5, row=0, sticky='e')


		### initializations and bindings
		self.signalplot = None
		self.rec_thread = None
		self.rec_exit_event = threading.Event()
		self.play_thread = None
		self.play_exit_event = threading.Event()
		self.playall = True
		self.protocol('WM_DELETE_WINDOW', self.cancel_pressed)

		self.bind('<KeyPress-Escape>', self.cancel_pressed)
		# remove the default behavior of invoking the button with the space key
		self.unbind_class('Button', '<Key-space>')

		# initial instruction
		if self.is_edit and self.type == 'sv':
			snd = AudioSegment.from_wav(self.aset.tmp_file)
			if snd.duration_seconds > 3.05:
				self.set_instruction(_("Click and drag the red box to select a stable section of the vowel.") +\
					'\n\n' + _("Verify with [Play Selection] / <P> or [Play All] / <Shift+P>.") +\
					'\n' + _("(Abort playback with <Esc>.)") +\
					'\n\n' + _("To accept the selected part of the voice sample for analysis press [Accept] / <A>."))
			else:
				self.set_instruction(_("{Duration of this vowel sample is 3 sec. or less; can't edit.}") +\
					'\n\n' + _("Verify with [Play Selection] / <P> or [Play All] / <Shift+P>.") +\
					'\n' + _("(Abort playback with <Esc>.)") +\
					'\n\n' + _("To accept the selected part of the voice sample for analysis press [Accept] / <A>."))
		elif self.is_edit and self.type == 'cs':
			self.set_instruction(_("Click (and drag) near the start/end of the red box to adjust the selection to the target utterance.") +\
				'\n\n' + _("Verify with [Play Selection] / <P> or [Play All] / <Shift+P>.") +\
				'\n' + _("(Abort playback with <Esc>.)") +\
				'\n\n' + _("To accept the selected part of the voice sample for analysis press [Accept] / <A>."))
		else:
			self.set_instruction(_("Press [Record] / <Space> to start recording."))

		self.signalplot = signalplot.Plotter(self.type, self.plot_box, self.aset, self.uset, large=True)
		self.signalplot.init_plot(recdur_factor=1, reset=True)

		if self.is_edit:
			self.toogle_rec_btn.state(['disabled'])
		else:
			self.toogle_playA_btn.state(['disabled'])
			self.toogle_playS_btn.state(['disabled'])
			self.rm_save_btn.state(['disabled'])
			# self.bind('<KeyPress-r>', self.start_recording)
			self.bind('<KeyPress-space>', self.start_recording)


	def set_instruction(self, content):
		nlines = content.count('\n') + 1
		if nlines == 1:
			content = '\n\n' + content
		elif nlines == 2:
			content = '\n' + content
		self.instr_text.configure(state='normal')
		self.instr_text.delete('1.0', 'end')
		self.instr_text.insert('1.0', content)
		# search for buttons to tag: [...]
		idx = '1.0'
		while True:
			sidx = self.instr_text.search(r'[\[]', idx, 'end', regexp=True)
			if not sidx: break
			eidx = self.instr_text.search(r'[\]]', sidx, 'end', regexp=True)
			if not eidx: break
			self.instr_text.delete(sidx, sidx + '+1c')
			self.instr_text.insert(sidx, ' ')
			self.instr_text.delete(eidx, eidx + '+1c')
			self.instr_text.insert(eidx, ' ')
			self.instr_text.tag_add('btn', sidx, eidx + '+1c')
			idx = eidx
		# search for keys to tag: <...>
		idx = '1.0'
		while True:
			sidx = self.instr_text.search(r'[<]', idx, 'end', regexp=True)
			if not sidx: break
			eidx = self.instr_text.search(r'[>]', sidx, 'end', regexp=True)
			if not eidx: break
			self.instr_text.delete(sidx, sidx + '+1c')
			self.instr_text.insert(sidx, ' ')
			self.instr_text.delete(eidx, eidx + '+1c')
			self.instr_text.insert(eidx, ' ')
			self.instr_text.tag_add('key', sidx, eidx + '+1c')
			idx = eidx
		# search for errors/warnings to tag: {...}
		idx = '1.0'
		while True:
			sidx = self.instr_text.search(r'[{}]', idx, 'end', regexp=True)
			if not sidx: break
			eidx = self.instr_text.search(r'[}]', sidx, 'end', regexp=True)
			if not eidx: break
			self.instr_text.delete(sidx, sidx + '+1c')
			self.instr_text.insert(sidx, ' ')
			self.instr_text.delete(eidx, eidx + '+1c')
			self.instr_text.insert(eidx, ' ')
			self.instr_text.tag_add('warn', sidx, eidx + '+1c')
			idx = eidx
		self.instr_text.tag_add('center', '1.0', 'end')
		self.instr_text.configure(state='disabled')


	def edit_sound(self):
		self.bind('<KeyPress-p>', lambda e: self.start_playing(playall=False))
		self.bind('<Shift-KeyPress-P>', lambda e: self.start_playing(playall=True))
		self.bind('<KeyPress-a>', self.save_pressed)
		self.signalplot.post_recording(self.aset.tmp_file, is_edit=True)


	def start_playing(self, playall):
		self.play_exit_event.clear()
		self.playall = playall
		self.unbind('<KeyPress-Escape>')
		self.bind('<KeyPress-Escape>', self.stop_playing)
		self.toogle_rec_btn.state(['disabled'])
		self.toogle_playA_btn.state(['disabled'])
		self.toogle_playS_btn.state(['disabled'])
		self.rm_save_btn.state(['disabled'])
		self.rm_cancel_btn.state(['disabled'])
		self.play_thread = threading.Thread(target=self.player)
		self.update_idletasks()
		self.play_thread.start()


	def stop_playing(self, *args):
		self.play_exit_event.set()


	def after_playing(self):
		self.play_thread = None
		self.unbind('<KeyPress-Escape>')
		self.bind('<KeyPress-Escape>', self.cancel_pressed)
		self.toogle_rec_btn.state(['!disabled'])
		self.toogle_playA_btn.state(['!disabled'])
		self.toogle_playS_btn.state(['!disabled'])
		self.rm_save_btn.state(['!disabled'])
		self.rm_cancel_btn.state(['!disabled'])


	def player(self):
		fname = self.aset.tmp_file
		chunk = self.aset.chunk
		wavefile = wave.open(fname, 'rb')
		selected_frames = wavefile.getnframes()
		if not self.playall:
			wavefile.setpos(int(self.signalplot.l_marker))
			selected_frames = int(self.signalplot.r_marker) - int(self.signalplot.l_marker)
		pa = pyaudio.PyAudio()
		stream = pa.open(
			format=pa.get_format_from_width(wavefile.getsampwidth()),
			channels=wavefile.getnchannels(),
			rate=wavefile.getframerate(),
			output=True
			)
		data = wavefile.readframes(chunk)
		frames_done = chunk
		while len(data) > 0 and frames_done <= selected_frames and not self.play_exit_event.is_set():
			stream.write(data)
			if frames_done + chunk > selected_frames:
				frames_to_read = selected_frames - frames_done
				data = wavefile.readframes(frames_to_read)
				frames_done = frames_done + frames_to_read
			else:
				data = wavefile.readframes(chunk)
				frames_done += chunk

		wavefile.close()
		stream.stop_stream()
		stream.close()
		pa.terminate()
		self.after_playing()


	def start_recording(self, *args):
		_ = self.aset.translate
		self.rec_exit_event.clear()
		# self.bind('<KeyPress-r>', self.stop_recording)
		if self.type == 'cs':
			self.heading_label.configure(text = _("Read this text:"))
		elif self.type == 'sv':
			self.heading_label.configure(text = _("Sustain vowel /a/ for at least 3 seconds."))
		self.bind('<KeyPress-space>', self.stop_recording)
		self.toogle_rec_btn.configure(command = self.stop_recording)
		self.toogle_rec_btn['text'] = _("Stop")
		self.set_instruction(_("Press [Stop] / <Space> to stop recording."))
		# clear and reset plot
		self.signalplot.init_plot(recdur_factor=1, reset=True)
		self.signalplot.figure.canvas.draw()
		self.rec_thread = threading.Thread(target=self.recorder)
		self.rec_thread.start()


	def stop_recording(self, *args):
		self.rec_exit_event.set()


	def after_recording(self):
		_ = self.aset.translate
		self.rec_thread = None
		# self.bind('<KeyPress-r>', self.start_recording)
		self.bind('<KeyPress-space>', self.start_recording)
		self.toogle_rec_btn.configure(command = self.start_recording)
		self.toogle_rec_btn['text'] = _("Record")
		snd = AudioSegment.from_wav(self.aset.tmp_file)
		if self.type == 'sv' and snd.duration_seconds < (self.aset.v_seconds_analyzed - 0.05):
			self.set_instruction(_("{This sample is too short!}") +\
				'\n\n' +\
				_("Regular analysis requires vowel duration = 3 seconds."))
			return
		self.toogle_playA_btn.state(['!disabled'])
		self.toogle_playS_btn.state(['!disabled'])
		self.rm_save_btn.state(['!disabled'])
		if self.type == 'cs':
			self.heading_label.configure(text = _("Select this text:"))
		elif self.type == 'sv':
			self.heading_label.configure(text = _("Select 3 seconds of stable section of the vowel."))
		self.bind('<KeyPress-p>', lambda e: self.start_playing(playall=False))
		self.bind('<Shift-KeyPress-P>', lambda e: self.start_playing(playall=True))
		self.bind('<KeyPress-a>', self.save_pressed)
		if self.type == 'cs':
			self.set_instruction(_("Click (and drag) near the start/end of the red box to adjust the selection to the target utterance.") +\
				'\n\n' + _("Verify with [Play Selection] / <P> or [Play All] / <Shift+P>.") +\
				'\n' + _("(Abort playback with <Esc>.)") +\
				'\n\n' + _("To accept the selected part of the voice sample for analysis press [Accept] / <A>.") +\
				'\n\n' + _("To reject and record a new voice sample press [Record] / <Space>."))
		elif self.type == 'sv':
			self.set_instruction(_("Click and drag the red box to select a stable section of the vowel.") +\
				'\n\n' + _("Verify with [Play Selection] / <P> or [Play All] / <Shift+P>.") +\
				'\n' + _("(Abort playback with <Esc>.)") +\
				'\n\n' + _("To accept the selected part of the voice sample for analysis press [Accept] / <A>.") +\
				'\n\n' + _("To reject and record a new voice sample press [Record] / <Space>."))
		time.sleep(0.5)
		self.signalplot.post_recording(self.aset.tmp_file)


	def recorder(self):
		fname = self.aset.tmp_file
		if os.path.exists(fname):
			os.remove(fname)
		channels = self.aset.channels
		srate = self.aset.sampling_rate
		sformat = pyaudio.paInt16
		chunk = self.aset.chunk
		pa = pyaudio.PyAudio()
		if self.type == 'sv':
			duration = self.uset.rec_duration['sv']
		elif self.type == 'cs':
			duration = self.uset.rec_duration['cs']
		wavefile = wave.open(fname, 'wb')
		wavefile.setnchannels(channels)
		wavefile.setsampwidth(pa.get_sample_size(sformat))
		wavefile.setframerate(srate)
		stream = pa.open(
			format=sformat,
			channels=channels,
			rate=srate,
			input=True,
			frames_per_buffer=chunk
			)
		frames_done = 0
		frame_count = 0
		while frame_count <= int((srate / chunk) * duration) and not self.rec_exit_event.is_set():
			# collect frames for wav file
			audio = stream.read(chunk)
			wavefile.writeframes(audio)
			# users can deactivate continuous live plot to reduce computing demands
			if self.uset.live_plot:
				frames_done += chunk
				audio_data = np.frombuffer(audio, np.int16)
				rstart = frames_done - chunk
				rstop = rstart + chunk
				self.signalplot.subplot.plot(
					range(rstart, rstop, 1),
					audio_data,
					color=(0,0,0,0.5),
					zorder=1
				)
				self.signalplot.figure.canvas.draw()
			frame_count += 1
		wavefile.close()
		stream.stop_stream()
		stream.close()
		pa.terminate()
		self.after_recording()


	def save_pressed(self, *args):
		_ = self.aset.translate
		app_win = self.aset.app_win
		cs_file = self.aset.cs_file
		sv_file = self.aset.sv_file
		sound = AudioSegment.from_wav(self.aset.tmp_file)
		# acknowledge user's selection
		trimed_sound = self.trim(sound)
		if self.type == 'cs':
			trimed_sound.export(cs_file, format='wav')
			app_win.main_cs_plot.plot_sound(cs_file)
			app_win.cs_play_btn.state(['!disabled'])
			app_win.cs_edit_btn.state(['!disabled'])
			app_win.cs_date_entry.state(['!disabled'])
			app_win.cs_save_btn.state(['!disabled'])
			if not self.is_edit:
				app_win.cs_filename.set(_("unsaved data"))
				app_win.update_sample_date(cs_file, 'cs')
		elif self.type == 'sv':
			trimed_sound.export(sv_file, format='wav')
			app_win.main_sv_plot.plot_sound(sv_file)
			app_win.sv_play_btn.state(['!disabled'])
			app_win.sv_edit_btn.state(['!disabled'])
			app_win.sv_date_entry.state(['!disabled'])
			app_win.sv_save_btn.state(['!disabled'])
			if not self.is_edit:
				app_win.sv_filename.set(_("unsaved data"))
				app_win.update_sample_date(sv_file, 'sv')
			self.aset.sv_too_long = False
			self.aset.sv_too_short = False
		self.reset_stuff()
		app_win.analyze_btn.state(['!disabled'])
		# app_win.corpus_btn.state(['disabled'])
		self.destroy()


	def cancel_pressed(self, *args):
		self.reset_stuff()
		self.destroy()


	def trim(self, sound):
		# trim sound to user's selection
		sr = self.aset.sampling_rate
		vsa = self.aset.v_seconds_analyzed
		if self.type == 'cs':
			right = (int(self.signalplot.r_marker) / sr) * 1000
			left = (int(self.signalplot.l_marker) / sr) * 1000
			tmp = sound[:right]
			ms = len(tmp) - left
			trimed_sound = tmp[-ms:]
		elif self.type == 'sv' and sound.duration_seconds > vsa:
			right = (self.signalplot.r_marker / sr) * 1000
			tmp = sound[:right]
			ms_analyzed = vsa * 1000
			trimed_sound = tmp[-ms_analyzed:]
		else:
			trimed_sound = sound
		return trimed_sound


	def reset_stuff(self):
		# reset recording stuff?
		pass









