# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



from matplotlib import gridspec, cm, rcParams
from matplotlib.projections import register_projection
from matplotlib.projections.polar import PolarAxes
from matplotlib.patches import Circle, RegularPolygon
from matplotlib.path import Path
from matplotlib.spines import Spine
from matplotlib.transforms import Affine2D
import numpy as np
import wave



def visualize(aset, a_lang, figure, metadata, params, samples):
	'''Configure and create results page

	:param aset: Object, reference to AppSettings instance (~global variables)
	:param a_lang: Int, index of target analysis language
	:param figure: Object, reference to figure instance
	:param metadata: Dict, input form data
	:param params: Dict, numerical results
	:param samples: String, type of sample(s) (sv/cs/mx)
	'''
	_ = aset.translate
	normv = aset.norm_values
	
	# set font sizes
	if aset.screen_height >= 1190:
		'''100% Din A4'''
		font_s = 6
		font_m = 9
		font_l = 14
		font_xl = 18
	elif aset.screen_height >= 800:
		'''75% Din A4'''
		font_s = 4
		font_m = 6
		font_l = 9
		font_xl = 12
	else:
		'''50% Din A4'''
		font_s = 3
		font_m = 4
		font_l = 7
		font_xl = 9

	# set font family
	# rcParams['font.family'] = 'sans-serif'
	# rcParams['font.sans-serif'] = ['Helvetica', 'Arial', 'Tahoma', 'DejaVu Sans', 'Lucida Grande', 'Verdana']

	### Layout
	gs = gridspec.GridSpec(
		5, # rows
		3, # columns
		height_ratios=[4,5,1,1,8],
		width_ratios=[1,10,5],
		figure=figure
		)
	mdata = figure.add_subplot(gs[0, 1:])
	spec_mx = figure.add_subplot(gs[1, 1])
	signal_mx = figure.add_subplot(gs[2, 1])
	vqadata = figure.add_subplot(gs[1:3, 2])
	# spider subplot is specified in the spider section below

# Lorem ipsum dolor sit amet, consetetur sadipscing elitr.

	### Title and metadata
	figure.suptitle("VOXplot – " + _("Acoustic Voice Quality Profile"), fontsize=font_xl)
	mdata.axis('off')
	mdata.text(0.5, 1, "VOXplot v" + aset.app_version, va='center', ha='center', fontsize=font_s)
	mdata.text(0.1, 0.7, _("Name:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.15, 0.7, metadata['title'] + metadata['name'], va='center', ha='left', fontsize=font_m)
	mdata.text(0.1, 0.6, _("ID:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.15, 0.6, metadata['subid'], va='center', ha='left', fontsize=font_m)
	mdata.text(0.1, 0.5, _("DoB:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.15, 0.5, metadata['dob'], va='center', ha='left', fontsize=font_m)
	mdata.text(0.1, 0.4, _("Language:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.15, 0.4, aset.analysis_langs[a_lang], va='center', ha='left', fontsize=font_m)
	mdata.text(0.1, 0.3, _("Notes:"), va='top', ha='right', fontsize=font_m, color='#666666')

	# notes: insert line breaks
	words = metadata['notes'].split()
	for w in words:
		if len(w) > 40:
			i = words.index(w)
			del words[i]
			while len(w) > 40:
				nw = w[:40]
				w = w[40:]
				words.insert(i, nw)
				i += 1
			else:
				words.insert(i, w)
	lines = []
	line = ""
	n = 0
	for w in words:
		if len(line) + len(w) <= 40:
			line = line + w + " "
		else:
			lines.append(line)
			line = ""
			line = line + w + " "
	lines.append(line)
	dots = False
	while len(lines) > 4:
		lines.pop()
		dots = True
	if dots: lines.append("...")
	metadata['notes'] = "\n".join(lines)
	mdata.text(0.15, 0.3, metadata['notes'], va='top', ha='left', fontsize=font_m)

	mdata.text(0.8, 0.7, _("Date:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.85, 0.7, metadata['date'], va='center', ha='left', fontsize=font_m)
	mdata.text(0.8, 0.6, _("Time:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.85, 0.6, metadata['time'], va='center', ha='left', fontsize=font_m)
	mdata.text(0.8, 0.5, _("Examiner:"), va='center', ha='right', fontsize=font_m, color='#666666')
	mdata.text(0.85, 0.5, metadata['examiner'], va='center', ha='left', fontsize=font_m)
	if samples != 'sv':
		mdata.text(0.8, 0.4, _("CS recording date:"), va='center', ha='right', fontsize=font_m, color='#666666')
		mdata.text(0.85, 0.4, metadata['csdate'], va='center', ha='left', fontsize=font_m)
	if samples != 'cs':
		mdata.text(0.8, 0.3, _("SV recording date:"), va='center', ha='right', fontsize=font_m, color='#666666')
		mdata.text(0.85, 0.3, metadata['svdate'], va='center', ha='left', fontsize=font_m)


	### Numerical data
	vqadata.axis('off')
	vqadata.set_ylim(-0.5, 16)
	if samples != 'cs':
		# headings
		vqadata.text(0.5, 16, "   Norm", va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
		vqadata.text(0.2, 15, "Slope (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 14, "Tilt (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 13, "H1H2 (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 12, "Jitter local (%):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 11, "Jitter ppq5 (%):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 10, "Shimmer (%):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 9, "Shimmer (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 8, "PSD (ms):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 7, "HNR (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 6, "HNR-D (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 5, "GNE:", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 4, "HF noise (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 3, "CPPS (dB):", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 2, "Voice breaks:", va='center', ha='right', fontsize=font_m)
		vqadata.text(0.2, 0.5, "AVQI:", va='center', ha='right', fontsize=font_m, fontweight='bold', color='#947700')
		vqadata.text(0.2, -0.5, "ABI:", va='center', ha='right', fontsize=font_m, fontweight='bold', color='#577FA7')

		# values
		vqadata.text(0.5, 15, '{:.2f}'.format(params['sv_slope']), va='center', ha='right', fontsize=font_m, fontweight='bold')
		vqadata.text(0.5, 15, "      —", va='center', ha='left', fontsize=font_m, color='g')
		vqadata.text(0.5, 14, '{:.2f}'.format(params['sv_tilt']), va='center', ha='right', fontsize=font_m, fontweight='bold')
		vqadata.text(0.5, 14, "      —", va='center', ha='left', fontsize=font_m, color='g')
		vqadata.text(0.5, 13, '{:.2f}'.format(params['sv_h1h2']), va='center', ha='right', fontsize=font_m, fontweight='bold')
		vqadata.text(0.5, 13, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_jitterLocal'] < 1000000:
			vq_jittlo = vqadata.text(0.5, 12, '{:.2f}'.format(params['sv_jitterLocal']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['jitterLocal'] != None:
				vqadata.text(0.5, 12, '   < ' + '{:.2f}'.format(normv['jitterLocal']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_jitterLocal'] > normv['jitterLocal']:
					vq_jittlo.set_color('r')
			else:
				vqadata.text(0.5, 12, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_jitterPPQ5'] < 1000000:
			vq_jittlo = vqadata.text(0.5, 11, '{:.2f}'.format(params['sv_jitterPPQ5']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['jitterPPQ5'] != None:
				vqadata.text(0.5, 11, '   < ' + '{:.2f}'.format(normv['jitterPPQ5']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_jitterPPQ5'] > normv['jitterPPQ5']:
					vq_jittlo.set_color('r')
			else:
				vqadata.text(0.5, 11, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_shimmerLocal'] < 1000000:
			vq_shimlo = vqadata.text(0.5, 10, '{:.2f}'.format(params['sv_shimmerLocal']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['shimmerLocal'] != None:
				vqadata.text(0.5, 10, '   < ' + '{:.2f}'.format(normv['shimmerLocal']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_shimmerLocal'] > normv['shimmerLocal']:
					vq_shimlo.set_color('r')
			else:
				vqadata.text(0.5, 10, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_shimmerLocaldB'] < 1000000:
			vq_shimdb = vqadata.text(0.5, 9, '{:.2f}'.format(params['sv_shimmerLocaldB']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['shimmerLocaldB'] != None:
				vqadata.text(0.5, 9, '   < ' + '{:.2f}'.format(normv['shimmerLocaldB']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_shimmerLocaldB'] > normv['shimmerLocaldB']:
					vq_shimdb.set_color('r')
			else:
				vqadata.text(0.5, 9, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_psd'] < 1000000:
			vq_psd = vqadata.text(0.5, 8, '{:.2f}'.format(params['sv_psd']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['psd'] != None:
				vqadata.text(0.5, 8, '   < ' + '{:.2f}'.format(normv['psd']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_psd'] > normv['psd']:
					vq_psd.set_color('r')
			else:
				vqadata.text(0.5, 8, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_hnr'] < 1000000:
			vq_hnr = vqadata.text(0.5, 7, '{:.2f}'.format(params['sv_hnr']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['hnr'] != None:
				vqadata.text(0.5, 7, '   > ' + '{:.2f}'.format(normv['hnr']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_hnr'] < normv['hnr']:
					vq_hnr.set_color('r')
			else:
				vqadata.text(0.5, 7, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_hnrd'] < 1000000:
			vq_hnrd = vqadata.text(0.5, 6, '{:.2f}'.format(params['sv_hnrd']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['hnrd'] != None:
				vqadata.text(0.5, 6, '   > ' + '{:.2f}'.format(normv['hnrd']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_hnrd'] < normv['hnrd']:
					vq_hnrd.set_color('r')
			else:
				vqadata.text(0.5, 6, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_gne'] < 1000000:
			vq_gne = vqadata.text(0.5, 5, '{:.2f}'.format(params['sv_gne']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['gne'] != None:
				vqadata.text(0.5, 5, '   > ' + '{:.2f}'.format(normv['gne']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_gne'] < normv['gne']:
					vq_gne.set_color('r')
			else:
				vqadata.text(0.5, 5, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_hfno'] < 1000000:
			vq_hfno = vqadata.text(0.5, 4, '{:.2f}'.format(params['sv_hfno']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['hfno'] != None:
				vqadata.text(0.5, 4, '   > ' + '{:.2f}'.format(normv['hfno']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_hfno'] < normv['hfno']:
					vq_hfno.set_color('r')
			else:
				vqadata.text(0.5, 4, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_cpps'] < 1000000:
			vq_cpps = vqadata.text(0.5, 3, '{:.2f}'.format(params['sv_cpps']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['cpps'] != None:
				vqadata.text(0.5, 3, '   > ' + '{:.2f}'.format(normv['cpps']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_cpps'] < normv['cpps']:
					vq_cpps.set_color('r')
			else:
				vqadata.text(0.5, 3, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['sv_voiceBreaks'] < 1000000:
			vq_voib = vqadata.text(0.5, 2, '{:.2f}'.format(params['sv_voiceBreaks']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['voiceBreaks'] != None:
				vqadata.text(0.5, 2, '   < ' + '{:.2f}'.format(normv['voiceBreaks']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['sv_voiceBreaks'] > normv['voiceBreaks']:
					vq_voib.set_color('r')
			else:
				vqadata.text(0.5, 2, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['avqi'] < 1000000:
			vq_avqi = vqadata.text(0.5, 0.5, '{:.2f}'.format(params['avqi']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['avqi'] != None:
				vqadata.text(0.5, 0.5, '   < ' + '{:.2f}'.format(normv['avqi']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['avqi'] > normv['avqi']:
					vq_avqi.set_color('r')
			else:
				vqadata.text(0.5, 0.5, "      —", va='center', ha='left', fontsize=font_m, color='g')
		if params['abi'] < 1000000:
			vq_abi = vqadata.text(0.5, -0.5, '{:.2f}'.format(params['abi']), va='center', ha='right', fontsize=font_m, fontweight='bold')
			if normv['abi'] != None:
				vqadata.text(0.5, -0.5, '   < ' + '{:.2f}'.format(normv['abi']), va='center', ha='left', fontsize=font_m, fontstyle='italic', color='g')
				if params['abi'] > normv['abi']:
					vq_abi.set_color('r')
			else:
				vqadata.text(0.5, -0.5, "      —", va='center', ha='left', fontsize=font_m, color='g')


	### Spectrogram and signal
	if samples != 'cs':
		# signal
		soundfile = wave.open(aset.sv_file, 'rb')
		swidth = soundfile.getsampwidth()
		rate = soundfile.getframerate()
		nframes = soundfile.getnframes()
		rawdata = soundfile.readframes(nframes)
		soundfile.close()
		audio_data = np.frombuffer(rawdata, dtype=np.int16)
		signal_mx.plot(
			range(0, nframes, 1),
			audio_data,
			color=(0.4,0.4,0.4,1)
			)
		signal_mx.set_yticks([0])
		signal_mx.set_yticklabels(['0'])
		xtick_labels_int = [l for l in range(60)]
		xtick_labels = list(map(str, xtick_labels_int))
		seconds = int(nframes/rate)
		signal_mx.set_xlim(0, nframes)
		signal_mx.set_xticks(range(0, (seconds + 1) * rate, rate))
		signal_mx.set_xticklabels(xtick_labels[0:seconds + 1])
		signal_mx.tick_params(axis='both', which='major', labelsize=font_s)
		signal_mx.tick_params(axis='y', which='both', left=True, right=True, labelleft=True, labelright=True)

		# pre-windowing of audio_data to determine dynamic dB range of spectrogram (max dB - xy)
		win_width = int(rate*0.05)
		step = int(0.01*rate)
		window = np.hamming(win_width)
		win_frames = []
		n_win_frames = 1+int((len(audio_data)-win_width)/step)
		for t in range (0, n_win_frames):
			win_frames.append(np.copy(audio_data[(t*step):(t*step+win_width)])*window)
		# set dynamic dB range
		vmin = 10*np.log10(np.max(win_frames)) - 40
		
		# create spectrogram
		spec_mx.specgram(
			audio_data,
			Fs=rate,
			NFFT=win_width,
			noverlap=int((rate*0.05)-(rate*0.002)),
			window=window,
			cmap=cm.gray_r,
			vmin=vmin
			# pad_to=2048,
			)
		spec_mx.set_ylim((0,5000))
		spec_mx.tick_params(axis='both', which='major', labelsize=font_s)
		spec_mx.tick_params(axis='x', bottom=False, top=False, labelbottom=False)
		spec_mx.tick_params(axis='y', which='both', left=True, right=True, labelleft=True, labelright=True)


	### Spider
	if samples != 'cs':
		param_names = [
			'AVQI',
			'HNR',
			'Jitter ppq5',
			'CPPS',
			'GNE',
			'ABI'
			]
		high_is_better = [
			False,
			True,
			False,
			True,
			True,
			False
			]
		norm = [
			normv['avqi'],
			normv['hnr'],
			normv['jitterPPQ5'],
			normv['cpps'],
			normv['gne'],
			normv['abi']
			]
		maxvals = [10, 36, 6, 23, 1, 10]
		minvals = [0, 1, 0, 2, 0.22, 0]
		avqi = (params['avqi'] if (params['avqi'] < 2000000 and params['avqi'] > 0) else 0)
		abi = (params['abi'] if (params['abi'] < 2000000 and params['abi'] > 0) else 0)
		data = [
			avqi,
			params['sv_hnr'],
			params['sv_jitterPPQ5'],
			params['sv_cpps'],
			params['sv_gne'],
			abi
			]

		theta = _init_spider(6, font_m, font_l, avqi, abi)
		spider_plot = figure.add_subplot(gs[4, 1:], projection='spider_projection')
		norm_n = _normalize_data(norm, norm, maxvals, minvals, high_is_better, spider_plot, theta)
		data_n = _normalize_data(data, norm, maxvals, minvals, high_is_better, spider_plot, theta)

		spider_plot.set_rlim(0,1.25,emit=True,auto=False)
		spider_plot.set_rgrids([0.25, 0.5, 0.75, 1.0], labels=[])
		spider_plot.plot(theta, data_n, color='r', alpha=1, linewidth=0.5)
		spider_plot.fill(theta, data_n, facecolor='r', alpha=1)
		spider_plot.plot(theta, norm_n, color='g', alpha=0.9, linewidth=0.5)
		spider_plot.fill(theta, norm_n, facecolor='g', alpha=0.70)
		spider_plot.set_param_labels(param_names)
		spider_plot.text(
			# theta[2]+(0.5*(theta[3]-theta[2])),
			theta[0]+(0.5*(theta[1]-theta[0])),
			1.3,
			_("Hoarseness"),
			horizontalalignment='right',
			verticalalignment='center',
			color='#947700',
			weight='bold',
			size=font_l,
			rotation=0)
		spider_plot.text(
			# theta[8]+(0.5*(theta[9]-theta[8])),
			theta[4]+(0.5*(theta[5]-theta[4])),
			1.3,
			_("Breathiness"),
			horizontalalignment='left',
			verticalalignment='center',
			color='#577FA7',
			weight='bold',
			size=font_l,
			rotation=0)



def _init_spider(num_vars, font_m, font_l, avqi, abi, frame='circle'):
	theta = np.linspace(0, 2*np.pi, num_vars, endpoint=False)

	class SpiderAxes(PolarAxes):
		name = 'spider_projection'

		def __init__(self, *args, **kwargs):
			super().__init__(*args, **kwargs)
			self.set_theta_zero_location('N', offset=30)
			# don't draw outer circle
			self.set_frame_on(False)

		def fill(self, *args, closed=True, **kwargs):
			return super().fill(closed=closed, *args, **kwargs)

		def plot(self, *args, **kwargs):
			lines = super().plot(*args, **kwargs)
			for line in lines:
				self._close_line(line)

		def set_param_labels(self, labels):
			(lines,labels) = self.set_thetagrids(np.degrees(theta), labels, fontsize=font_m)
			for i in range(1,3):
				labels[i].set_color('#947700')
				labels[i].set_fontweight('bold')
			for i in range(3,5):
				labels[i].set_color('#577FA7')
				labels[i].set_fontweight('bold')
			labels[0].set_fontweight('bold')
			labels[0].set_fontsize(font_m)
			labels[0].set_color('w')
			if avqi == 0:
				# sv analysis without cs --> hide AVQI label
				labels[0].set_backgroundcolor('w')
			else:
				labels[0].set_backgroundcolor('#947700')
			labels[5].set_fontweight('bold')
			labels[5].set_fontsize(font_m)
			labels[5].set_color('w')
			if abi == 0:
				# sv analysis without cs --> hide ABI label
				labels[5].set_backgroundcolor('w')
			else:
				labels[5].set_backgroundcolor('#577FA7')

		def _close_line(self, line):
			x, y = line.get_data()
			if x[0] != x[-1]:
				x = np.append(x, x[0])
				y = np.append(y, y[0])
				line.set_data(x, y)

		def _gen_axes_patch(self):
			if frame == 'circle':
				return Circle((0.5, 0.5), 0.5)
			elif frame == 'polygon':
				return RegularPolygon((0.5, 0.5), num_vars, radius=.5, edgecolor="k")

		def _gen_axes_spines(self):
			if frame == 'circle':
				return super()._gen_axes_spines()
			elif frame == 'polygon':
				spine = Spine(axes=self, spine_type='circle', path=Path.unit_regular_polygon(num_vars))
				spine.set_transform(Affine2D().scale(.5).translate(.5, .5) + self.transAxes)
				return {'polar': spine}

	register_projection(SpiderAxes)
	return theta



def _normalize_data(data, norm, mx, mn, hib, ax, theta):
	idx = [0,1,2,3,4,5]
	normalized = []
	for d, no, mx, mn, hib, i in zip(data, norm, mx, mn, hib, idx):
		if hib and no and mx:
			### higher values are better (HNR, GNE, CPPS)
			n = (((mx - d) - (mx - no)) / ((no - mn) / 0.75)) + 0.25
		elif not hib and no and mx:
			### higher values are worse (AVQI, ABI, Jitter)
			### algorithm:
			# 0..mx = 100%
			# norm set to 25% --> mx-norm = 75%
			# calculate percentage of data
			n = ((d - no) / ((mx - no) / 0.75)) + 0.25
		else:
			n = 0

		n = (0.05 if n < 0.05 else n)
		if n > 1.0:
			n = 1.025
			ax.annotate(
				'',
				xy=(theta[i], 1.15),
				xytext=(theta[i], 0.99),
				arrowprops=dict(arrowstyle="->, head_length=0.2,head_width=0.2", color='r', linewidth=1.0))
		normalized.append(n)
	return normalized



