# -*- coding: utf-8 -*-

#    VOXplot
#    
#    Copyright: Jörg Mayer
#    Email: joerg.mayer@lingphon.com
#    Web: https://voxplot.lingphon.com
#    
#    VOXplot is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#    
#    VOXplot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    
#    You should have received a copy of the GNU General Public
#    License along with VOXplot (cf. COPYING). If not, see
#    https://www.gnu.org/licenses/gpl-3.0.en.html



from matplotlib.figure import Figure
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg as FigureCanvas
from matplotlib.patches import Rectangle
import numpy as np
import wave



class Plotter(object):
	'''Handle plotting and interaction with plots.

	:param type: String, type of signal (cs or sv)
	:param target: Object, reference to target frame
	:param ASet: Object, reference to AppSettings instance (~global variables)
	:param USet: Object, reference to UserSettings instance
	:param large: Boolean, plot size large (True) or small (False)
	'''
	def __init__(self, type, target, ASet, USet, large=False):
		self.type = type
		self.aset = ASet
		self.uset = USet
		if large:
			self.figure = Figure(tight_layout=True, figsize=(9,2))
		else:
			self.figure = Figure(tight_layout=True, figsize=(6,1.7))
		self.subplot = self.figure.add_subplot(111)
		self.canvas = FigureCanvas(self.figure, master=target)
		self.canvas.get_tk_widget().grid(column=0, row=0)
		xtick_labels_int = []
		self.xtick_labels = []
		self.cs_sensitive_area = 0
		self._reset()


	### public methods: plot and initialize interactions
	def init_plot(self, new_recdur=0, recdur_factor=1, rate=0, reset=True):
		if rate == 0: rate = self.aset.sampling_rate
		if recdur_factor == 1: self.subplot.cla()
		if new_recdur > 0: self.recdur = new_recdur
		self.recdur = self.recdur * recdur_factor
		self.subplot.set_xlim(0, self.recdur * rate)
		seconds = int(self.recdur)
		if seconds < 20:
			xtick_factor = 1
			xtick_labels_int = [l for l in range(0,seconds + 1,1)]
		else:
			xtick_factor = 2
			xtick_labels_int = [l for l in range(0,seconds + 1,2)]
		xtick_labels = list(map(str, xtick_labels_int))
		self.subplot.set_xticks(range(0, (seconds + 1) * rate, rate * xtick_factor))
		self.subplot.set_xticklabels(xtick_labels[0:seconds + 1])
		self.subplot.set_yticks([0])
		self.subplot.set_yticklabels(['0'])
		if reset: self._reset()


	def plot_sound(self, sndfile):
		wf = wave.open(sndfile, 'rb')
		swidth = wf.getsampwidth()
		rate = wf.getframerate()
		nframes = wf.getnframes()
		rawdata = wf.readframes(nframes)
		wf.close()
		audio_data = np.frombuffer(rawdata, dtype=np.int16)
		recdur = nframes/rate
		# seconds = math.ceil(nframes/rate)
		self.init_plot(recdur, 1, rate)
		self.subplot.plot(
			range(0, nframes, 1),
			audio_data,
			color=(0,0,0,0.5),
			zorder=5
			)
		self.xMin,self.xMax = self.subplot.get_xlim()
		self.yMin,self.yMax = self.subplot.get_ylim()
		self.figure.canvas.draw()


	def post_recording(self, sndfile, is_edit=False):
		self.plot_sound(sndfile)
		sr = self.aset.sampling_rate
		vfa = self.aset.v_frames_analyzed
		# set markers to xmin and xmax
		self.l_marker = self.xMin
		self.r_marker = self.xMax
		duration = round((self.r_marker - self.l_marker) / sr, 2)
		if self.type == 'cs':
			if not is_edit:
				self.l_marker = self.xMin + sr
				self.r_marker = self.xMax - sr
			self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
			self.cs_rect = Rectangle(
				(self.l_marker, self.yMin),
				self.r_marker - self.l_marker,
				(self.yMin * -1) + self.yMax,
				linewidth=2,
				edgecolor='r',
				facecolor='none',
				zorder=100
				)
			self.subplot.add_patch(self.cs_rect)
			self.cs_rect.figure.canvas.draw()
			self.connect_id_press = self.canvas.mpl_connect('button_press_event', self._on_cs_canvas_press)
			self.connect_id_move = self.canvas.mpl_connect('motion_notify_event', lambda event: self._on_cs_canvas_move(event, sr))
			self.connect_id_release = self.canvas.mpl_connect('button_release_event', self._on_cs_canvas_release)
		elif self.type == 'sv' and duration > self.aset.v_seconds_analyzed:
			self.l_marker = (self.xMax * 0.5) - (self.aset.v_frames_analyzed * 0.5)
			self.r_marker = self.l_marker + self.aset.v_frames_analyzed
			self.sv_rect = Rectangle(
				(self.l_marker, self.yMin),
				self.aset.v_frames_analyzed,
				(self.yMin * -1) + self.yMax,
				linewidth=2,
				edgecolor='r',
				facecolor='none',
				zorder=100
				)
			self.subplot.add_patch(self.sv_rect)
			self.sv_rect.figure.canvas.draw()
			self.connect_id_press = self.canvas.mpl_connect('button_press_event', lambda event: self._on_sv_canvas_press(event, vfa))
			self.connect_id_move = self.canvas.mpl_connect('motion_notify_event', lambda event: self._on_sv_canvas_move(event, vfa))
			self.connect_id_release = self.canvas.mpl_connect('button_release_event', self._on_sv_canvas_release)
			# self.connect_id_key = self.canvas.mpl_connect('key_press_event', lambda event: self._on_sv_canvas_key(event, sr, vfa))


	### private methods: implement interactions
	### cs canvas
	def _on_cs_canvas_press(self, event):
		# click outside axes to move boundaries to start/end
		if not event.inaxes and event.x < 200:
			self.lock = 'start'
			self.cs_rect.set_x(0)
			self.l_marker = 0
			self.cs_rect.set_width(self.r_marker - 0)
			self.cs_rect.figure.canvas.draw()
			self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
			return
		elif not event.inaxes and event.x > 600:
			self.lock = 'end'
			self.cs_rect.set_width(self.xMax - self.l_marker)
			self.r_marker = self.xMax
			self.cs_rect.figure.canvas.draw()
			self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
			return
		# do nothing if clicked in the void between left and right boundary
		if event.xdata > self.l_marker + self.cs_sensitive_area and event.xdata < self.r_marker - self.cs_sensitive_area: return
		# jump left/right boundary to clicked position inside axes
		if event.xdata < self.l_marker + self.cs_sensitive_area:
			self.lock = 'start'
			self.cs_rect.set_x(event.xdata)
			self.l_marker = event.xdata
			self.cs_rect.set_width(self.r_marker - event.xdata)
			self.cs_rect.figure.canvas.draw()
		elif event.xdata > self.r_marker - self.cs_sensitive_area:
			self.lock = 'end'
			self.cs_rect.set_width(event.xdata - self.l_marker)
			self.r_marker = event.xdata
			self.cs_rect.figure.canvas.draw()
		# click and drag
		self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
		x0, y0 = self.cs_rect.xy
		width = self.cs_rect.get_width()
		self.press = x0, y0, width, event.xdata, event.ydata


	def _on_cs_canvas_move(self, event, sr):
		if event.xdata != None and event.xdata < self.l_marker + self.cs_sensitive_area and self.lock != 'end':
			self.aset.rec_win.configure(cursor='left_side')
			if self.press != None  and self.lock == 'start':
				x0, y0, width, xpress, ypress = self.press
				dx = event.xdata - xpress
				if x0+dx < self.xMin:
					newX = self.xMin
				elif x0+dx+sr >= self.r_marker:
					newX = self.r_marker - sr
				else:
					newX = x0+dx
				self.cs_rect.set_x(newX)
				self.l_marker = newX
				self.cs_rect.set_width(self.r_marker - newX)
				self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
				self.cs_rect.figure.canvas.draw()
		elif event.xdata != None and event.xdata > self.r_marker - self.cs_sensitive_area and self.lock != 'start':
			self.aset.rec_win.configure(cursor='right_side')
			if self.press != None and self.lock == 'end':
				x0, y0, width, xpress, ypress = self.press
				dx = event.xdata - xpress
				newW = width + dx
				if x0+newW > self.xMax:
					newW = self.xMax - x0
				elif x0+newW <= x0 + sr:
					newW = sr
				self.cs_rect.set_width(newW)
				self.r_marker = x0 + newW
				self.cs_sensitive_area = (self.r_marker - self.l_marker) * 0.4
				self.cs_rect.figure.canvas.draw()
		else:
			self.aset.rec_win.configure(cursor='arrow')


	def _on_cs_canvas_release(self, event):
		self.aset.rec_win.configure(cursor='arrow')
		self.press = None
		self.lock = ''
		self.cs_rect.figure.canvas.draw()


	### sv canvas
	def _on_sv_canvas_press(self, event, vfa):
		# click outside axes to move boundaries to start/end
		if not event.inaxes and event.x < 200:
			self.sv_rect.set_x(0)
			self.sv_rect.figure.canvas.draw()
			self.l_marker = 0
			self.r_marker = vfa
			return
		elif not event.inaxes and event.x > 600:
			self.sv_rect.set_x(self.xMax - vfa)
			self.sv_rect.figure.canvas.draw()
			self.l_marker = self.xMax - vfa
			self.r_marker = self.xMax
			return
		# jump left/right boundary to clicked position inside axes
		if event.inaxes and event.xdata < self.l_marker:
			self.sv_rect.set_x(event.xdata)
			self.sv_rect.figure.canvas.draw()
			self.l_marker = event.xdata
			self.r_marker = event.xdata + vfa
			return
		elif event.inaxes and event.xdata > self.r_marker:
			self.sv_rect.set_x(event.xdata - vfa)
			self.sv_rect.figure.canvas.draw()
			self.l_marker = event.xdata - vfa
			self.r_marker = event.xdata
			return
		# click and drag
		contains, attrd = self.sv_rect.contains(event)
		if not contains: return
		x0, y0 = self.sv_rect.xy
		self.press = x0, y0, event.xdata, event.ydata


	def _on_sv_canvas_move(self, event, vfa):
		if event.xdata != None and event.xdata > self.l_marker and event.xdata < self.r_marker:
			self.aset.rec_win.configure(cursor='sb_h_double_arrow')
		elif event.xdata != None and event.xdata < self.l_marker:
			self.aset.rec_win.configure(cursor='left_side')
		elif event.xdata != None and event.xdata > self.r_marker:
			self.aset.rec_win.configure(cursor='right_side')
		else:
			self.aset.rec_win.configure(cursor='arrow')
		if self.press is None: return
		if event.inaxes != self.sv_rect.axes: return
		x0, y0, xpress, ypress = self.press
		dx = event.xdata - xpress
		if x0+dx < self.xMin:
			newX = self.xMin
		elif x0+dx+vfa > self.xMax:
			newX = self.xMax - vfa
		else:
			newX = x0+dx
		self.sv_rect.set_x(newX)
		self.sv_rect.figure.canvas.draw()
		self.l_marker = newX
		self.r_marker = self.l_marker + vfa


	def _on_sv_canvas_release(self, event):
		self.press = None
		self.sv_rect.figure.canvas.draw()


	def _on_sv_canvas_key(self, event, sr, vfa):
		vfa = self.aset.v_frames_analyzed
		key_pressed = event.key
		x0, y0 = self.sv_rect.xy
		dx = sr / 100
		if key_pressed == 'left':
			if x0-dx < self.xMin:
				newX = self.xMin
			else:
				newX = x0-dx
			self.sv_rect.set_x(newX)
			self.l_marker = newX
			self.r_marker = self.l_marker + vfa
			self.sv_rect.figure.canvas.draw()
		elif key_pressed == 'right':
			if x0+dx+vfa > self.xMax:
				newX = self.xMax - vfa
			else:
				newX = x0+dx
			self.sv_rect.set_x(newX)
			self.l_marker = newX
			self.r_marker = self.l_marker + vfa
			self.sv_rect.figure.canvas.draw()
		else:
			return


	### helper method
	def _reset(self):
		if self.type == 'cs':
			self.recdur = self.uset.rec_duration['cs']
		elif self.type == 'sv':
			self.recdur = self.uset.rec_duration['sv']
		self.xMin = None
		self.xMax = None
		self.yMin = None
		self.yMax = None
		self.l_marker = 0
		self.r_marker = 0
		self.sv_rect = None
		self.cs_rect = None
		self.connect_id_press = None
		self.connect_id_move = None
		self.connect_id_release = None
		self.connect_id_key = None
		self.press = None
		self.lock = ''


